import os
import unittest
import zipfile

from conans.client.tools import environment_append
from cpt.test.utils.tools import TestClient, TestServer

from cpt.test.test_client.tools import get_patched_multipackager


class ConfigInstallTest(unittest.TestCase):

    conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    pass
"""

    def test_toolsets_works(self):

        ts = TestServer(users={"user": "password"})
        tc = TestClient(servers={"default": ts}, users={"default": [("user", "password")]})
        tc.save({"conanfile.py": self.conanfile})

        zip_path = os.path.join(tc.current_folder, 'config.zip')
        zipf = zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED)
        zipf.close()

        with environment_append({"CONAN_UPLOAD": ts.fake_url, "CONAN_LOGIN_USERNAME": "user",
                                 "CONAN_PASSWORD": "password", "CONAN_USERNAME": "user",
                                 "CONAN_CONFIG_URL": zip_path}):
            mulitpackager = get_patched_multipackager(tc, exclude_vcvars_precommand=True)
            mulitpackager.add_common_builds(reference="lib/1.0@user/stable",
                                            shared_option_name=False)
            mulitpackager.run()
            self.assertIn("Installing config from address %s" % zip_path, tc.out)



import os.path
from conans import tools
from conans.model.conf import ConfDefinition


class ConfigManager(object):

    def __init__(self, conan_api, printer):
        self._conan_api = conan_api
        self.printer = printer

    def install(self, url, args=None):
        message = "Installing config from address %s" % url
        if args:
            message += " with args \"%s\"" % args
        self.printer.print_message(message)
        self._conan_api.config_install(url, verify_ssl=True, args=args)


class GlobalConf(object):
    def __init__(self, conan_api, printer):
        self._conan_api = conan_api
        self.printer = printer

    def populate(self, values):
        global_conf = self._conan_api.app.cache.new_config_path
        if isinstance(values, str):
            values = values.split(",")
        config = ConfDefinition()
        if os.path.exists(global_conf):
            content = tools.load(global_conf)
            config.loads(content)
        for value in values:
            key = value[:value.find('=')]
            k_value = value[value.find('=') + 1:]
            config.update(key, k_value)
        tools.save(global_conf, config.dumps())



import unittest
import textwrap

from conans.client.tools import environment_append, load
from cpt.test.utils.tools import TestClient

from cpt.test.test_client.tools import get_patched_multipackager


class GlobalConfTest(unittest.TestCase):

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            pass
        """)

    def test_environment_variable(self):
        tc = TestClient(users={"default": [("user", "password")]})
        tc.save({"conanfile.py": self.conanfile})
        global_conf = ["tools.system.package_manager:mode=install", "tools.system.package_manager:sudo=True"]

        with environment_append({"CONAN_GLOBAL_CONF": ",".join(global_conf)}):
            mulitpackager = get_patched_multipackager(tc, exclude_vcvars_precommand=True)
            mulitpackager.add_common_builds(reference="lib/1.0@user/stable", shared_option_name=False)
            mulitpackager.run()
            assert textwrap.dedent("""tools.system.package_manager:mode=install
            tools.system.package_manager:sudo=True
            """.replace(" ", "")) == load(tc.cache.new_config_path)



import os


def get_bool_from_env(var_name):
    val = os.getenv(var_name, None)
    return str(val).lower() not in ("0", "none", "false")


def get_custom_bool_from_env(var_name, default=None):
    val = os.getenv(var_name, default)
    return str(val).lower() in ('true', 'on', 'ok', 'y', 'yes', '1')


def split_colon_env(varname):
    if os.getenv(varname) is None:
        return None
    if os.getenv(varname).strip() == "":
        return []
    return [a.strip() for a in list(filter(None, os.getenv(varname, "").split(",")))]


def transform_list_options_to_dict(list_options):
    assert isinstance(list_options, list)
    dict_options = {}
    for option in list_options:
        if '=' not in option:
            raise RuntimeError("Option %s does not contain '='" % option)
        option_obj = option.split('=')
        dict_options[option_obj[0]] = option_obj[1]
    return dict_options



import six
from conans.client.conan_api import Conan
from conans.model.version import Version
from cpt import get_client_version

from cpt.packager import ConanMultiPackager


def get_patched_multipackager(tc, *args, **kwargs):
    client_version = get_client_version()
    extra_init_kwargs = {}
    if Version("1.11") < Version(client_version) < Version("1.18"):
        extra_init_kwargs.update({'requester': tc.requester})
    elif Version(client_version) >= Version("1.18"):
        extra_init_kwargs.update({'http_requester': tc.requester})

    if Version(client_version) < Version("1.12.0"):
        cache = tc.client_cache
    else:
        cache = tc.cache

    conan_api = Conan(cache_folder=cache.cache_folder, output=tc.out, **extra_init_kwargs)

    class Printer(object):

        def __init__(self, tc):
            self.tc = tc

        def __call__(self, contents):
            if six.PY2:
                contents = unicode(contents)
            self.tc.out.write(contents)

    kwargs["out"] = Printer(tc)
    kwargs["conan_api"] = conan_api
    kwargs["cwd"] = tc.current_folder

    mp = ConanMultiPackager(*args, **kwargs)
    return mp



import os
import platform
import re
import sys
import copy
from collections import defaultdict
from itertools import product

import six
from conans import tools
from conans.client.conan_api import Conan
from conans.client.runner import ConanRunner
from conans.model.ref import ConanFileReference
from conans.model.version import Version

from cpt import get_client_version
from cpt.auth import AuthManager
from cpt.builds_generator import BuildConf, BuildGenerator
from cpt.ci_manager import CIManager
from cpt.printer import Printer
from cpt.profiles import get_profiles, save_profile_to_tmp
from cpt.remotes import RemotesManager
from cpt.runner import CreateRunner, DockerCreateRunner
from cpt.tools import get_bool_from_env, get_custom_bool_from_env
from cpt.tools import split_colon_env
from cpt.uploader import Uploader
from cpt.config import ConfigManager


def load_cf_class(path, conan_api):
    client_version = get_client_version()
    client_version = Version(client_version)
    if client_version < Version("1.7.0"):
        from conans.client.loader_parse import load_conanfile_class
        return load_conanfile_class(path)
    elif client_version < Version("1.14.0"):
        return conan_api._loader.load_class(path)
    elif client_version < Version("1.15.0"):
        remotes = conan_api._cache.registry.remotes.list
        for remote in remotes:
            conan_api.python_requires.enable_remotes(remote_name=remote)
        return conan_api._loader.load_class(path)
    elif client_version < Version("1.16.0"):
        remotes = conan_api._cache.registry.load_remotes()
        conan_api.python_requires.enable_remotes(remotes=remotes)
        return conan_api._loader.load_class(path)
    elif client_version < Version("1.18.0"):
        remotes = conan_api._cache.registry.load_remotes()
        conan_api._python_requires.enable_remotes(remotes=remotes)
        return conan_api._loader.load_class(path)
    else:
        if not conan_api.app:
            conan_api.create_app()
        remotes = conan_api.app.cache.registry.load_remotes()
        conan_api.app.python_requires.enable_remotes(remotes=remotes)
        if client_version < Version("1.20.0"):
            return conan_api.app.loader.load_class(path)
        elif client_version < Version("1.21.0"):
            return conan_api.app.loader.load_basic(path)
        else:
            conan_api.app.pyreq_loader.enable_remotes(remotes=remotes)
            return conan_api.app.loader.load_named(path, None, None, None, None)


class PlatformInfo(object):
    """Easy mockable for testing"""
    @staticmethod
    def system():
        import platform
        return platform.system()


class ConanOutputRunner(ConanRunner):

    def __init__(self):
        super(ConanOutputRunner, self).__init__()

        class OutputInternal(object):
            def __init__(self):
                self.output = ""

            def write(self, data):
                self.output += str(data)
                sys.stdout.write(data)

        self._output = OutputInternal()

    @property
    def output(self):
        return self._output.output

    def __call__(self, command):
        return super(ConanOutputRunner, self).__call__(command, output=self._output)


class ConanMultiPackager(object):
    """ Help to generate common builds (setting's combinations), adjust the environment,
    and run conan create command in docker containers"""

    def __init__(self, username=None, channel=None, runner=None,
                 gcc_versions=None, visual_versions=None, visual_runtimes=None,
                 visual_toolsets=None,
                 apple_clang_versions=None,
                 msvc_versions=None, msvc_runtimes=None, msvc_runtime_types=None,
                 archs=None, options=None,
                 use_docker=None, curpage=None, total_pages=None,
                 docker_image=None, reference=None, password=None,
                 remotes=None,
                 upload=None, stable_branch_pattern=None,
                 vs10_x86_64_enabled=False,
                 mingw_configurations=None,
                 stable_channel=None,
                 platform_info=None,
                 upload_retry=None,
                 clang_versions=None,
                 login_username=None,
                 upload_only_when_stable=None,
                 upload_only_when_tag=None,
                 upload_only_recipe=None,
                 upload_force=None,
                 build_types=None,
                 cppstds=None,
                 skip_check_credentials=False,
                 allow_gcc_minors=False,
                 exclude_vcvars_precommand=False,
                 docker_run_options=None,
                 docker_image_skip_update=False,
                 docker_image_skip_pull=False,
                 docker_entry_script=None,
                 docker_32_images=None,
                 docker_conan_home=None,
                 docker_shell=None,
                 pip_install=None,
                 build_policy=None,
                 require_overrides=None,
                 always_update_conan_in_docker=False,
                 conan_api=None,
                 client_cache=None,
                 conanfile=None,
                 ci_manager=None,
                 out=None,
                 test_folder=None,
                 cwd=None,
                 config_url=None,
                 config_args=None,
                 upload_dependencies=None,
                 force_selinux=None,
                 skip_recipe_export=False,
                 update_dependencies=None,
                 lockfile=None,
                 global_conf=None,
                 version_alias=None):

        conan_version = get_client_version()

        self.printer = Printer(out)
        self.printer.print_rule()
        self.printer.print_ascci_art()

        self.cwd = cwd or os.getcwd()

        if not conan_api:
            self.conan_api, _, _ = Conan.factory()
            self.conan_api.create_app()
            self.client_cache = self.conan_api.app.cache
        else:
            self.conan_api = conan_api
            self.client_cache = client_cache

        self.ci_manager = ci_manager or CIManager(self.printer)
        self.remotes_manager = RemotesManager(self.conan_api, self.printer, remotes, upload)
        self.username = username or os.getenv("CONAN_USERNAME", None)

        self.skip_check_credentials = skip_check_credentials or get_bool_from_env("CONAN_SKIP_CHECK_CREDENTIALS")

        self.auth_manager = AuthManager(self.conan_api, self.printer, login_username, password,
                                        default_username=self.username,
                                        skip_check_credentials=self.skip_check_credentials)

        # Upload related variables
        self.upload_retry = upload_retry or os.getenv("CONAN_UPLOAD_RETRY", 3)

        if upload_only_when_stable is not None:
            self.upload_only_when_stable = upload_only_when_stable
        else:
            self.upload_only_when_stable = get_bool_from_env("CONAN_UPLOAD_ONLY_WHEN_STABLE")

        if upload_only_when_tag is not None:
            self.upload_only_when_tag = upload_only_when_tag
        else:
            self.upload_only_when_tag = get_bool_from_env("CONAN_UPLOAD_ONLY_WHEN_TAG")

        self.upload_only_recipe = upload_only_recipe or get_bool_from_env("CONAN_UPLOAD_ONLY_RECIPE")
        self.upload_force = upload_force if upload_force is not None \
                            else get_custom_bool_from_env("CONAN_UPLOAD_FORCE", True)

        self.remotes_manager.add_remotes_to_conan()
        self.uploader = Uploader(self.conan_api, self.remotes_manager, self.auth_manager,
                                 self.printer, self.upload_retry, self.upload_force)

        self._builds = []
        self._named_builds = {}
        self._packages_summary = []

        self._update_conan_in_docker = always_update_conan_in_docker or get_bool_from_env("CONAN_ALWAYS_UPDATE_CONAN_DOCKER")

        self._platform_info = platform_info or PlatformInfo()

        self.stable_branch_pattern = stable_branch_pattern or \
                                     os.getenv("CONAN_STABLE_BRANCH_PATTERN",
                                               "master$ main$ release.* stable.*")

        self.stable_channel = stable_channel or os.getenv("CONAN_STABLE_CHANNEL", "stable")
        self.stable_channel = self.stable_channel.rstrip()
        self.partial_reference = reference or os.getenv("CONAN_REFERENCE", None)
        self.channel = self._get_specified_channel(channel, reference)
        self.conanfile = conanfile or os.getenv("CONAN_CONANFILE", "conanfile.py")
        self.version_alias = version_alias or os.getenv("CPT_VERSION_ALIAS", None)

        if self.partial_reference:
            if "@" in self.partial_reference:
                self.reference = ConanFileReference.loads(self.partial_reference)
            else:
                name, version = self.partial_reference.split("/")
                self.reference = ConanFileReference(name, version, self.username, self.channel)
        else:
            if not os.path.exists(os.path.join(self.cwd, self.conanfile)):
                raise Exception("Conanfile not found, specify a 'reference' "
                                "parameter with name and version")

            conanfile = load_cf_class(os.path.join(self.cwd, self.conanfile), self.conan_api)
            name, version = conanfile.name, conanfile.version
            if name and version:
                self.reference = ConanFileReference(name, version, self.username, self.channel)
            else:
                self.reference = None

        self._docker_image = docker_image or os.getenv("CONAN_DOCKER_IMAGE", None)

        # If CONAN_DOCKER_IMAGE is specified, then use docker is True
        self.use_docker = (use_docker or get_bool_from_env("CONAN_USE_DOCKER") or
                           self._docker_image is not None)

        self.docker_conan_home = docker_conan_home or os.getenv("CONAN_DOCKER_HOME", None)

        os_name = self._platform_info.system() if not self.use_docker else "Linux"
        self.build_generator = BuildGenerator(reference, os_name, gcc_versions,
                                              apple_clang_versions, clang_versions,
                                              visual_versions, visual_runtimes, visual_toolsets,
                                              vs10_x86_64_enabled,
                                              msvc_versions, msvc_runtimes, msvc_runtime_types,
                                              mingw_configurations, archs, allow_gcc_minors,
                                              build_types, options, cppstds)

        self.build_policy = (build_policy or
                        self.ci_manager.get_commit_build_policy() or
                        split_colon_env("CONAN_BUILD_POLICY"))
        if isinstance(self.build_policy, list):
            self.build_policy = ",".join(self.build_policy)

        self.require_overrides = require_overrides or split_colon_env("CONAN_REQUIRE_OVERRIDES") or None

        self.sudo_docker_command = ""
        if "CONAN_DOCKER_USE_SUDO" in os.environ:
            self.sudo_docker_command = "sudo -E" if get_bool_from_env("CONAN_DOCKER_USE_SUDO") else ""
        elif platform.system() != "Windows":
            self.sudo_docker_command = "sudo -E"

        self.sudo_pip_command = ""
        if "CONAN_PIP_USE_SUDO" in os.environ:
            self.sudo_pip_command = "sudo -E" if get_bool_from_env("CONAN_PIP_USE_SUDO") else ""
        elif platform.system() != "Windows" and self._docker_image and 'conanio/' not in str(self._docker_image):
            self.sudo_pip_command = "sudo -E"
        self.pip_command = os.getenv("CONAN_PIP_COMMAND", "pip")
        pip_found = True if tools.os_info.is_windows else tools.which(self.pip_command)
        if not pip_found or not "pip" in self.pip_command:
            raise Exception("CONAN_PIP_COMMAND: '{}' is not a valid pip command.".format(self.pip_command))
        self.docker_pip_command = os.getenv("CONAN_DOCKER_PIP_COMMAND", "pip")

        self.docker_shell = docker_shell or os.getenv("CONAN_DOCKER_SHELL")

        if self.is_wcow:
            if self.docker_conan_home is None:
                self.docker_conan_home = "C:/Users/ContainerAdministrator"
                self.docker_shell = docker_shell or "cmd /C"
        else:
            if self.docker_conan_home is None:
                self.docker_conan_home = "/home/conan"
                self.docker_shell = docker_shell or "/bin/sh -c"

        self.docker_platform_param = ""
        self.lcow_user_workaround = ""

        if self.is_lcow:
            self.docker_platform_param = "--platform=linux"
            # With LCOW, Docker doesn't respect USER directive in dockerfile yet
            self.lcow_user_workaround = "sudo su conan && "

        self.exclude_vcvars_precommand = exclude_vcvars_precommand or \
                                          get_bool_from_env("CONAN_EXCLUDE_VCVARS_PRECOMMAND")
        self._docker_image_skip_update = docker_image_skip_update or \
                                          get_bool_from_env("CONAN_DOCKER_IMAGE_SKIP_UPDATE")
        self._docker_image_skip_pull = docker_image_skip_pull or \
                                        get_bool_from_env("CONAN_DOCKER_IMAGE_SKIP_PULL")

        self.runner = runner or os.system
        self.output_runner = ConanOutputRunner()

        self.docker_run_options = docker_run_options or split_colon_env("CONAN_DOCKER_RUN_OPTIONS")
        if isinstance(self.docker_run_options, list):
            self.docker_run_options = " ".join(self.docker_run_options)

        self.docker_entry_script = docker_entry_script or os.getenv("CONAN_DOCKER_ENTRY_SCRIPT")

        self.pip_install = pip_install or split_colon_env("CONAN_PIP_INSTALL")

        self.upload_dependencies = upload_dependencies or split_colon_env("CONAN_UPLOAD_DEPENDENCIES") or ""
        if isinstance(self.upload_dependencies, list):
            self.upload_dependencies = ",".join(self.upload_dependencies)
        if "all" in self.upload_dependencies and self.upload_dependencies != "all":
            raise Exception("Upload dependencies only accepts or 'all' or package references. Do not mix both!")

        self.update_dependencies = update_dependencies or get_bool_from_env("CONAN_UPDATE_DEPENDENCIES")

        if self.channel:
            os.environ["CONAN_CHANNEL"] = self.channel

        if docker_32_images is not None:
            self.docker_32_images = docker_32_images
        else:
            self.docker_32_images = os.getenv("CONAN_DOCKER_32_IMAGES", False)

        self.force_selinux = force_selinux or get_bool_from_env("CONAN_FORCE_SELINUX")
        self.curpage = curpage or os.getenv("CONAN_CURRENT_PAGE", 1)
        self.total_pages = total_pages or os.getenv("CONAN_TOTAL_PAGES", 1)

        self.conan_pip_package = os.getenv("CONAN_PIP_PACKAGE", "conan==%s" % conan_version)
        if self.conan_pip_package in ("0", "False"):
            self.conan_pip_package = ""
        self.vs10_x86_64_enabled = vs10_x86_64_enabled

        self.builds_in_current_page = []

        self.global_conf = global_conf or os.getenv("CONAN_GLOBAL_CONF")

        self.test_folder = test_folder or os.getenv("CPT_TEST_FOLDER")

        self.config_url = config_url or os.getenv("CONAN_CONFIG_URL")

        self.skip_recipe_export = skip_recipe_export or \
                                     get_bool_from_env("CONAN_SKIP_RECIPE_EXPORT")
        self.config_args = config_args or os.getenv("CONAN_CONFIG_ARGS")

        self.lockfile = lockfile or os.getenv("CONAN_LOCKFILE")

        def valid_pair(var, value):
            return (isinstance(value, six.string_types) or
                    isinstance(value, bool) or
                    isinstance(value, list)) and not var.startswith("_") and "password" not in var
        with self.printer.foldable_output("local_vars"):
            self.printer.print_dict({var: value
                                     for var, value in self.__dict__.items()
                                     if valid_pair(var, value)})

    # For Docker on Windows, including Linux containers on Windows
    @property
    def is_lcow(self):
        return self.container_os == "linux" and platform.system() == "Windows"

    @property
    def is_wcow(self):
        return self.container_os == "windows" and platform.system() == "Windows"

    @property
    def container_os(self):
        # CONAN_DOCKER_PLATFORM=linux must be specified for LCOW
        if self.use_docker:
            if "CONAN_DOCKER_PLATFORM" in os.environ:
                return os.getenv("CONAN_DOCKER_PLATFORM", "windows").lower()
            else:
                return "windows"
        else:
            return ""

    @property
    def packages_summary(self):
        return self._packages_summary

    def save_packages_summary(self, file):
        self.printer.print_message("Saving packages summary to " + file)
        import json
        import datetime
        def default(o):
            if isinstance(o, (datetime.date, datetime.datetime)):
                return o.isoformat()

        with open(file, 'w') as outfile:
            json.dump(self.packages_summary, outfile, default = default)

    @property
    def items(self):
        return self._builds

    @items.setter
    def items(self, confs):
        self.builds = confs

    @property
    def builds(self):
        # Retrocompatibility iterating
        self.printer.print_message("WARNING",
                                   "\n\n\n******* ITERATING THE CONAN_PACKAGE_TOOLS BUILDS WITH "
                                   ".builds is deprecated use '.items' instead (unpack 5 elements: "
                                   "settings, options, env_vars, build_requires, reference  *******"
                                   "**\n\n\n")
        return [elem[0:4] for elem in self._builds]

    @builds.setter
    def builds(self, confs):
        """For retro compatibility directly assigning builds"""
        self._named_builds = {}
        self._builds = []
        for values in confs:
            if len(values) == 2:
                self._builds.append(BuildConf(values[0], values[1], {}, {}, self.reference))
            elif len(values) == 4:
                self._builds.append(BuildConf(values[0], values[1], values[2], values[3],
                                              self.reference))
            elif len(values) != 5:
                raise Exception("Invalid build configuration, has to be a tuple of "
                                "(settings, options, env_vars, build_requires, reference)")
            else:
                self._builds.append(BuildConf(*values))

    @property
    def named_builds(self):
        return self._named_builds

    @named_builds.setter
    def named_builds(self, confs):
        self._builds = []
        self._named_builds = {}
        for key, pages in confs.items():
            for values in pages:
                if len(values) == 2:
                    bc = BuildConf(values[0], values[1], {}, {}, self.reference)
                    self._named_builds.setdefault(key, []).append(bc)
                elif len(values) == 4:
                    bc = BuildConf(values[0], values[1], values[2], values[3], self.reference)
                    self._named_builds.setdefault(key, []).append(bc)
                elif len(values) != 5:
                    raise Exception("Invalid build configuration, has to be a tuple of "
                                    "(settings, options, env_vars, build_requires, reference)")
                else:
                    self._named_builds.setdefault(key, []).append(BuildConf(*values))

    def login(self, remote_name):
        self.auth_manager.login(remote_name)

    def add_common_builds(self, shared_option_name=None, pure_c=None,
                          dll_with_static_runtime=False, reference=None, header_only=True,
                          build_all_options_values=None):
        if reference:
            if "@" in reference:
                reference = ConanFileReference.loads(reference)
            else:
                name, version = reference.split("/")
                reference = ConanFileReference(name, version, self.username, self.channel)
        else:
            reference = self.reference

        if not reference:
            raise Exception("Specify a CONAN_REFERENCE or name and version fields in the recipe")

        if shared_option_name is None:
            env_shared_option_name = os.getenv("CONAN_SHARED_OPTION_NAME", None)
            shared_option_name = env_shared_option_name if str(env_shared_option_name).lower() != "false" else False

        if pure_c is None:
            pure_c = get_custom_bool_from_env("CONAN_PURE_C", True)

        build_all_options_values = build_all_options_values or split_colon_env("CONAN_BUILD_ALL_OPTIONS_VALUES") or []
        if not isinstance(build_all_options_values, list):
            raise Exception("'build_all_options_values' must be a list. e.g. ['foo:opt', 'foo:bar']")

        conanfile = None
        if os.path.exists(os.path.join(self.cwd, self.conanfile)):
            conanfile = load_cf_class(os.path.join(self.cwd, self.conanfile), self.conan_api)

        header_only_option = None
        if conanfile:
            if hasattr(conanfile, "options") and conanfile.options and "header_only" in conanfile.options:
                header_only_option = "%s:header_only" % reference.name

        if shared_option_name is None:
            if conanfile:
                if hasattr(conanfile, "options") and conanfile.options and "shared" in conanfile.options:
                    shared_option_name = "%s:shared" % reference.name

        # filter only valid options
        raw_options_for_building = [opt[opt.find(":") + 1:] for opt in build_all_options_values]
        for raw_option in reversed(raw_options_for_building):
            if hasattr(conanfile, "options") and conanfile.options and \
               not isinstance(conanfile.options.get(raw_option), list):
                raw_options_for_building.remove(raw_option)
        if raw_options_for_building and conanfile:
            # get option and its values
            cloned_options = copy.copy(conanfile.options)
            for key, value in conanfile.options.items():
                if key == "shared" and shared_option_name:
                    continue
                elif key not in raw_options_for_building:
                    del cloned_options[key]
            cloned_options2 = {}
            for key, value in cloned_options.items():
                # add package reference to the option name
                if not key.startswith("{}:".format(reference.name)):
                    cloned_options2["{}:{}".format(reference.name, key)] = value
            # combine all options x values (cartesian product)
            build_all_options_values = [dict(zip(cloned_options2, v)) for v in product(*cloned_options2.values())]

        builds = self.build_generator.get_builds(pure_c, shared_option_name,
                                                 dll_with_static_runtime, reference,
                                                 build_all_options_values)

        if header_only_option and header_only:
            if conanfile.default_options.get("header_only"):
                cloned_builds = copy.deepcopy(builds)
                for settings, options, env_vars, build_requires, reference in cloned_builds:
                    options.update({header_only_option: False})
                builds.extend(cloned_builds)
            else:
                settings, options, env_vars, build_requires, reference = builds[0]
                cloned_options = copy.copy(options)
                cloned_options.update({header_only_option: True})
                builds.append(BuildConf(copy.copy(settings), cloned_options, copy.copy(env_vars),
                                        copy.copy(build_requires), reference))

        self._builds.extend(builds)

    def add(self, settings=None, options=None, env_vars=None, build_requires=None, reference=None):
        settings = settings or {}
        options = options or {}
        env_vars = env_vars or {}
        build_requires = build_requires or {}
        if reference:
            reference = ConanFileReference.loads("%s@%s/%s" % (reference,
                                                               self.username, self.channel))
        reference = reference or self.reference
        self._builds.append(BuildConf(settings, options, env_vars, build_requires, reference))

    def remove_build_if(self, predicate):
        filtered_builds = []
        for build in self.items:
            if not predicate(build):
                filtered_builds.append(build)

        self._builds = filtered_builds

    def update_build_if(self, predicate, new_settings=None, new_options=None, new_env_vars=None,
                        new_build_requires=None, new_reference=None):
        updated_builds = []
        for build in self.items:
            if predicate(build):
                if new_settings:
                    build.settings.update(new_settings)
                if new_options:
                    build.options.update(new_options)
                if new_build_requires:
                    build.build_requires.update(new_build_requires)
                if new_env_vars:
                    build.env_vars.update(new_env_vars)
                if new_reference:
                    build.reference = new_reference
            updated_builds.append(build)
        self._builds = updated_builds

    def run(self, base_profile_name=None, summary_file=None, base_profile_build_name=None):
        env_vars = self.auth_manager.env_vars()
        env_vars.update(self.remotes_manager.env_vars())
        with tools.environment_append(env_vars):
            self.printer.print_message("Running builds...")
            if self.ci_manager.skip_builds():
                self.printer.print_message("Skipped builds due [skip ci] commit message")
                return 99
            if not self.skip_check_credentials and self._upload_enabled():
                self.auth_manager.login(self.remotes_manager.upload_remote_name)
            if self.conan_pip_package and not self.use_docker:
                with self.printer.foldable_output("pip_update"):
                    self.runner('%s %s install -q %s' % (self.sudo_pip_command,
                                                      self.pip_command,
                                                      self.conan_pip_package))
                    if self.pip_install:
                        packages = " ".join(self.pip_install)
                        self.printer.print_message("Install extra python packages: {}".format(packages))
                        self.runner('%s %s install -q %s' % (self.sudo_pip_command,
                                                          self.pip_command, packages))

            self.run_builds(base_profile_name=base_profile_name,
                            base_profile_build_name=base_profile_build_name)

        summary_file = summary_file or os.getenv("CPT_SUMMARY_FILE", None)
        if summary_file:
            self.save_packages_summary(summary_file)

    def _upload_enabled(self):
        if not self.remotes_manager.upload_remote_name:
            return False

        if not self.auth_manager.credentials_ready(self.remotes_manager.upload_remote_name):
            return False

        if self.upload_only_when_tag and not self.ci_manager.is_tag():
            self.printer.print_message("Skipping upload, not tag branch")
            return False

        st_channel = self.stable_channel or "stable"
        if self.upload_only_when_stable and self.channel != st_channel and not self.upload_only_when_tag:
            self.printer.print_message("Skipping upload, not stable channel")
            return False

        if not os.getenv("CONAN_TEST_SUITE", False):
            if self.ci_manager.is_pull_request():
                # PENDING! can't found info for gitlab/bamboo
                self.printer.print_message("Skipping upload, this is a Pull Request")
                return False

        def raise_error(field):
            raise Exception("Upload not possible, '%s' is missing!" % field)

        if not self.channel and "@" not in self.partial_reference:
            raise_error("channel")
        if not self.username and "@" not in self.partial_reference:
            raise_error("username")

        return True

    def run_builds(self, curpage=None, total_pages=None, base_profile_name=None,
                   base_profile_build_name=None):
        if len(self.named_builds) > 0 and len(self.items) > 0:
            raise Exception("Both bulk and named builds are set. Only one is allowed.")

        self.builds_in_current_page = []
        if len(self.items) > 0:
            curpage = curpage or int(self.curpage)
            total_pages = total_pages or int(self.total_pages)
            for index, build in enumerate(self.items):
                if curpage is None or total_pages is None or (index % total_pages) + 1 == curpage:
                    self.builds_in_current_page.append(build)
        elif len(self.named_builds) > 0:
            curpage = curpage or self.curpage
            if curpage not in self.named_builds:
                raise Exception("No builds set for page %s" % curpage)
            for build in self.named_builds[curpage]:
                self.builds_in_current_page.append(build)

        self.printer.print_current_page(curpage, total_pages)
        self.printer.print_jobs(self.builds_in_current_page)

        pulled_docker_images = defaultdict(lambda: False)
        skip_recipe_export = False

        base_profile_build_name = base_profile_build_name or os.getenv("CONAN_BASE_PROFILE_BUILD")
        if base_profile_build_name is not None:
            if get_client_version() < Version("1.24.0"):
                raise Exception("Conan Profile Build requires >= 1.24")
            self.printer.print_message("**************************************************")
            self.printer.print_message("Using specified "
                                        "build profile: %s" % base_profile_build_name)
            self.printer.print_message("**************************************************")

        # FIXME: Remove in Conan 1.3, https://github.com/conan-io/conan/issues/2787
        for index, build in enumerate(self.builds_in_current_page):
            self.printer.print_message("Build: %s/%s" % (index+1, len(self.builds_in_current_page)))
            base_profile_name = base_profile_name or os.getenv("CONAN_BASE_PROFILE")
            if base_profile_name:
                self.printer.print_message("**************************************************")
                self.printer.print_message("Using specified default "
                                           "base profile: %s" % base_profile_name)
                self.printer.print_message("**************************************************")
                if self.config_url:
                    ConfigManager(self.conan_api, self.printer).install(url=self.config_url, args=self.config_args)

            profile_text, base_profile_text = get_profiles(self.client_cache, build,
                                                           base_profile_name)
            profile_build_text, base_profile_build_text = get_profiles(self.client_cache, build,
                                                      base_profile_build_name, True)
            if not self.use_docker:
                profile_abs_path = save_profile_to_tmp(profile_text)
                if base_profile_build_text:
                    profile_build_abs_path = save_profile_to_tmp(profile_build_text)
                else:
                    profile_build_abs_path = None
                r = CreateRunner(profile_abs_path, build.reference, self.conan_api,
                                 self.uploader,
                                 exclude_vcvars_precommand=self.exclude_vcvars_precommand,
                                 build_policy=self.build_policy,
                                 require_overrides=self.require_overrides,
                                 runner=self.runner,
                                 cwd=self.cwd,
                                 printer=self.printer,
                                 upload=self._upload_enabled(),
                                 upload_only_recipe=self.upload_only_recipe,
                                 test_folder=self.test_folder,
                                 config_url=self.config_url,
                                 config_args=self.config_args,
                                 upload_dependencies=self.upload_dependencies,
                                 conanfile=self.conanfile,
                                 lockfile=self.lockfile,
                                 skip_recipe_export=skip_recipe_export,
                                 update_dependencies=self.update_dependencies,
                                 profile_build_abs_path=profile_build_abs_path,
                                 global_conf=self.global_conf,
                                 version_alias=self.version_alias
                                 )
                r.run()
                self._packages_summary.append({"configuration":  build, "package" : r.results})
            else:
                if not base_profile_build_text:
                    profile_build_text = None
                docker_image = self._get_docker_image(build)
                r = DockerCreateRunner(profile_text, base_profile_text, base_profile_name,
                                       build.reference,
                                       conan_pip_package=self.conan_pip_package,
                                       docker_image=docker_image,
                                       sudo_docker_command=self.sudo_docker_command,
                                       sudo_pip_command=self.sudo_pip_command,
                                       docker_image_skip_update=self._docker_image_skip_update,
                                       docker_image_skip_pull=self._docker_image_skip_pull,
                                       build_policy=self.build_policy,
                                       require_overrides=self.require_overrides,
                                       always_update_conan_in_docker=self._update_conan_in_docker,
                                       upload=self._upload_enabled(),
                                       upload_retry=self.upload_retry,
                                       upload_only_recipe=self.upload_only_recipe,
                                       upload_force=self.upload_force,
                                       runner=self.runner,
                                       docker_shell=self.docker_shell,
                                       docker_conan_home=self.docker_conan_home,
                                       docker_platform_param=self.docker_platform_param,
                                       docker_run_options=self.docker_run_options,
                                       lcow_user_workaround=self.lcow_user_workaround,
                                       test_folder=self.test_folder,
                                       pip_install=self.pip_install,
                                       docker_pip_command=self.docker_pip_command,
                                       config_url=self.config_url,
                                       config_args=self.config_args,
                                       printer=self.printer,
                                       upload_dependencies=self.upload_dependencies,
                                       conanfile=self.conanfile,
                                       lockfile=self.lockfile,
                                       force_selinux=self.force_selinux,
                                       skip_recipe_export=skip_recipe_export,
                                       update_dependencies=self.update_dependencies,
                                       profile_build_text=profile_build_text,
                                       base_profile_build_text=base_profile_build_text,
                                       global_conf=self.global_conf,
                                       version_alias=self.version_alias,
                                       cwd=self.cwd)

                r.run(pull_image=not pulled_docker_images[docker_image],
                      docker_entry_script=self.docker_entry_script)
                pulled_docker_images[docker_image] = True

            skip_recipe_export = self.skip_recipe_export

    def _get_docker_image(self, build):
        if self._docker_image:
            docker_image = self._docker_image
        else:
            compiler_name = build.settings.get("compiler", "")
            compiler_version = build.settings.get("compiler.version", "")
            docker_image = self._autodetect_docker_base_image(compiler_name, compiler_version)

        arch = build.settings.get("arch", "") or build.settings.get("arch_build", "")
        if self.docker_32_images and arch == "x86":
            build.settings["arch_build"] = "x86"
            docker_arch_suffix = "x86"
        elif arch != "x86" and arch != "x86_64":
            docker_arch_suffix = arch
        else:
            docker_arch_suffix = None
        if docker_arch_suffix and "-" not in docker_image:
            docker_image = "%s-%s" % (docker_image, docker_arch_suffix)

        return docker_image

    @staticmethod
    def _autodetect_docker_base_image(compiler_name, compiler_version):
        if compiler_name not in ["clang", "gcc"]:
            raise Exception("Docker image cannot be autodetected for "
                            "the compiler %s" % compiler_name)

        if compiler_name == "gcc" and Version(compiler_version) > Version("5"):
            compiler_version = Version(compiler_version).major(fill=False)

        return "conanio/%s%s" % (compiler_name, compiler_version.replace(".", ""))

    def _get_channel(self, specified_channel, stable_channel, upload_when_tag):
        if not specified_channel:
            return

        branch = self.ci_manager.get_branch()
        self.printer.print_message("Branch detected", branch)

        for pattern in self.stable_branch_pattern.split(" "):
            prog = re.compile(pattern)

            if branch and prog.match(branch):
                self.printer.print_message("Info",
                                           "Redefined channel by CI branch matching with '%s', "
                                           "setting CONAN_CHANNEL to '%s'" % (pattern,
                                                                              stable_channel))
                return stable_channel

        if self.ci_manager.is_tag() and upload_when_tag:
            self.printer.print_message("Info",
                                           "Redefined channel by branch tag, "
                                           "setting CONAN_CHANNEL to '%s'" % stable_channel)
            return stable_channel

        return specified_channel

    def _get_specified_channel(self, channel, reference):
        partial_reference = reference or os.getenv("CONAN_REFERENCE", None)
        specified_channel = None
        # without name/channel e.g. zlib/1.2.11@
        if partial_reference:
            if "@" in partial_reference:
                specified_channel = channel or os.getenv("CONAN_CHANNEL", None)
            else:
                specified_channel = channel or os.getenv("CONAN_CHANNEL", "testing")
                specified_channel = specified_channel.rstrip()
        else:
            if self.username:
                specified_channel = channel or os.getenv("CONAN_CHANNEL", "testing")
                specified_channel = specified_channel.rstrip()
            else:
                specified_channel = channel or os.getenv("CONAN_CHANNEL", None)
        return self._get_channel(specified_channel, self.stable_channel, self.upload_only_when_tag)



import os

from conans import tools
from conans.client.conan_api import Conan
from conans.model.ref import ConanFileReference
from conans.model.version import Version

from cpt.auth import AuthManager
from cpt.printer import Printer
from cpt.profiles import save_profile_to_tmp
from cpt.remotes import RemotesManager
from cpt.runner import CreateRunner, unscape_env
from cpt.uploader import Uploader
from cpt import get_client_version


def run():
    conan_version = get_client_version()
    if conan_version < Version("1.18.0"):
        conan_api, client_cache, _ = Conan.factory()
    else:
        conan_api, _, _ = Conan.factory()
        conan_api.create_app()
        client_cache = conan_api.app.cache

    printer = Printer()

    remotes_manager = RemotesManager(conan_api, printer)
    remotes_manager.add_remotes_to_conan()
    default_username = os.getenv("CONAN_USERNAME")
    auth_manager = AuthManager(conan_api, printer, default_username=default_username)

    upload_retry = os.getenv("CPT_UPLOAD_RETRY")
    upload_only_recipe = os.getenv("CPT_UPLOAD_ONLY_RECIPE")
    upload_force = os.getenv("CPT_UPLOAD_FORCE")
    uploader = Uploader(conan_api, remotes_manager, auth_manager, printer, upload_retry, upload_force)
    build_policy = unscape_env(os.getenv("CPT_BUILD_POLICY"))
    require_overrides = unscape_env(os.getenv("CPT_REQUIRE_OVERRIDES"))
    test_folder = unscape_env(os.getenv("CPT_TEST_FOLDER"))
    global_conf = unscape_env(os.getenv("CPT_GLOBAL_CONF"))
    reference = ConanFileReference.loads(os.getenv("CONAN_REFERENCE"))

    profile_text = unscape_env(os.getenv("CPT_PROFILE"))
    abs_profile_path = save_profile_to_tmp(profile_text)
    base_profile_text = unscape_env(os.getenv("CPT_BASE_PROFILE"))
    profile_build_text = unscape_env(os.getenv("CPT_PROFILE_BUILD"))
    config_url = unscape_env(os.getenv("CPT_CONFIG_URL"))
    config_args = unscape_env(os.getenv("CPT_CONFIG_ARGS"))
    upload_dependencies = unscape_env(os.getenv("CPT_UPLOAD_DEPENDENCIES"))
    update_dependencies = unscape_env(os.getenv("CPT_UPDATE_DEPENDENCIES"))
    conanfile = unscape_env(os.getenv("CPT_CONANFILE"))
    lockfile = unscape_env(os.getenv("CPT_LOCKFILE"))
    skip_recipe_export = unscape_env(os.getenv("CPT_SKIP_RECIPE_EXPORT"))
    if base_profile_text:
        base_profile_name = unscape_env(os.getenv("CPT_BASE_PROFILE_NAME"))
        tools.save(os.path.join(client_cache.profiles_path, base_profile_name),
                   base_profile_text)
    if profile_build_text:
        abs_profile_build_path = save_profile_to_tmp(profile_build_text)
    else:
        abs_profile_build_path = None

    upload = os.getenv("CPT_UPLOAD_ENABLED")
    runner = CreateRunner(abs_profile_path, reference, conan_api, uploader,
                          build_policy=build_policy, require_overrides=require_overrides, printer=printer, upload=upload,
                          upload_only_recipe=upload_only_recipe,
                          test_folder=test_folder, config_url=config_url, config_args=config_args,
                          upload_dependencies=upload_dependencies, conanfile=conanfile,
                          skip_recipe_export=skip_recipe_export,
                          update_dependencies=update_dependencies,
                          lockfile=lockfile,
                          profile_build_abs_path=abs_profile_build_path,
                          global_conf=global_conf)
    runner.run()


if __name__ == '__main__':
    run()



import unittest

from cpt.test.utils.tools import TestClient
from cpt.test.test_client.tools import get_patched_multipackager


class RequireOverridesTest(unittest.TestCase):
    def test_require_overrides(self):
        conanfile_bar = """from conans import ConanFile
class Pkg(ConanFile):
    name = "bar"
    version = "0.1.0"

    def build(self):
        pass
        """

        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foobar"
    version = "2.0"
    requires = "bar/0.1.0@foo/stable"

    def build(self):
        self.output.warn("BUILDING")
    """

        client = TestClient()
        client.save({"conanfile_bar.py": conanfile_bar})
        client.run("export conanfile_bar.py foo/stable")
        client.run("export conanfile_bar.py foo/testing")

        client.save({"conanfile.py": conanfile})
        mulitpackager = get_patched_multipackager(client, username="user",
                                                           channel="testing",
                                                           build_policy="missing",
                                                           require_overrides=["bar/0.1.0@foo/testing"],
                                                           exclude_vcvars_precommand=True)
        mulitpackager.add({}, {})
        mulitpackager.run()

        self.assertIn("requirement bar/0.1.0@foo/stable overridden by your conanfile to bar/0.1.0@foo/testing", client.out)



import platform
import unittest

from conans.client.tools import environment_append
from cpt.test.utils.tools import TestClient, TestServer

from cpt.test.test_client.tools import get_patched_multipackager


class VisualToolsetsTest(unittest.TestCase):

    conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os", "compiler"

    def build(self):
        self.output.warn("HALLO")
"""

    def test_toolsets_works(self):

        ts = TestServer(users={"user": "password"})
        tc = TestClient(servers={"default": ts}, users={"default": [("user", "password")]})
        tc.save({"conanfile.py": self.conanfile})
        with environment_append({"CONAN_UPLOAD": ts.fake_url, "CONAN_LOGIN_USERNAME": "user",
                                 "CONAN_PASSWORD": "password", "CONAN_USERNAME": "user",
                                 "CONAN_VISUAL_TOOLSETS": "17=v140;v140_xp,11=v140;v140_xp"}):
            mulitpackager = get_patched_multipackager(tc, exclude_vcvars_precommand=True)
            mulitpackager.add_common_builds(reference="lib/1.0@user/stable",
                                            shared_option_name=False)
            mulitpackager.run()
            if platform.system() == "Windows":
                self.assertIn("Uploading package 1/4", tc.out)
                self.assertIn("Uploading package 2/4", tc.out)
                self.assertIn("Uploading package 3/4", tc.out)
                self.assertIn("Uploading package 4/4", tc.out)
                self.assertIn("compiler.toolset=v140", tc.out)
                self.assertIn("compiler.toolset=v140_xp", tc.out)
            else:
                self.assertIn("Uploading package 1/2", tc.out)
                self.assertIn("Uploading package 2/2", tc.out)



import unittest
import textwrap
from cpt.config import GlobalConf
from cpt.printer import Printer
from cpt.test.unit.packager_test import MockConanAPI
from conans import tools


class GlobalConfUnitTest(unittest.TestCase):

    def setUp(self):
        self.conan_api = MockConanAPI()
        self.configuration = ["tools.system.package_manager:mode=install", "tools.system.package_manager:sudo=True"]

    def test_new_global_conf(self):
        manager = GlobalConf(self.conan_api, Printer())
        manager.populate(self.configuration)
        content = tools.load(self.conan_api._cache.new_config_path)
        assert content == textwrap.dedent("""tools.system.package_manager:mode=install
                                             tools.system.package_manager:sudo=True
                                          """.replace(" ", ""))

    def test_append_global_conf(self):
        manager = GlobalConf(self.conan_api, Printer())
        manager.populate(self.configuration)
        append_conf = ["tools.system.package_manager:tool=yum"]
        manager.populate(append_conf)
        content = tools.load(self.conan_api._cache.new_config_path)
        assert content == textwrap.dedent("""tools.system.package_manager:mode=install
                                             tools.system.package_manager:sudo=True
                                             tools.system.package_manager:tool=yum
                                          """.replace(" ", ""))

    def test_string_global_conf(self):
        configuration = "tools.system.package_manager:mode=install,tools.system.package_manager:sudo=True"
        manager = GlobalConf(self.conan_api, Printer())
        manager.populate(configuration)
        content = tools.load(self.conan_api._cache.new_config_path)
        assert content == textwrap.dedent("""tools.system.package_manager:mode=install
                                             tools.system.package_manager:sudo=True
                                          """.replace(" ", ""))



import os
import unittest
import sys
import json

from conans import tools
from conans.model.ref import ConanFileReference
from conans.errors import ConanException

from cpt.test.integration.base import BaseTest
from cpt.packager import ConanMultiPackager
from cpt.test.unit.utils import MockCIManager


class SimpleTest(BaseTest):

    def test_missing_full_reference(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    pass
"""
        self.save_conanfile(conanfile)
        mp = ConanMultiPackager(username="lasote")
        with self.assertRaisesRegexp(Exception, "Specify a CONAN_REFERENCE or name and version"):
            mp.add_common_builds()

    @unittest.skipUnless(sys.platform.startswith("win"), "Requires Windows")
    def test_visual(self):
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

    def build(self):
        assert("WindowsLibPath" in os.environ)

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           visual_versions=["17"],
                                           archs=["x86"],
                                           build_types=["Release"],
                                           visual_runtimes=["MD"],
                                           reference="zlib/1.2.2")
        self.packager.add_common_builds()
        self.packager.run_builds(1, 1)

    def test_visual_exclude_precommand(self):
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

    def build(self):
        assert("WindowsLibPath" not in os.environ)

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           visual_versions=["17"],
                                           archs=["x86"],
                                           build_types=["Release"],
                                           visual_runtimes=["MD"],
                                           exclude_vcvars_precommand=True,
                                           reference="zlib/1.2.2")
        self.packager.add_common_builds()
        self.packager.run_builds(1, 1)

    @unittest.skipUnless(sys.platform.startswith("win"), "Requires Windows")
    def test_msvc(self):
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

    def build(self):
        assert("LOCALAPPDATA" in os.environ)

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="user",
                                           channel="mychannel",
                                           msvc_versions=["193"],
                                           archs=["x86_64"],
                                           build_types=["Release"],
                                           cppstds=["14"],
                                           msvc_runtimes=["dynamic"],
                                           reference="zlib/1.2.2")
        self.packager.add_common_builds()
        self.packager.run_builds(1, 1)


    def test_shared_option_auto_managed(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote")
        self.packager.add_common_builds()
        self.assertIn("lib:shared", self.packager.items[0].options)

        # Even without name and version but reference
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}


"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote", reference="lib2/1.0")
        self.packager.add_common_builds()
        self.assertIn("lib2:shared", self.packager.items[0].options)

        self.packager = ConanMultiPackager(username="lasote", reference="lib2/1.0")
        self.packager.add_common_builds(shared_option_name=False)
        self.assertNotIn("lib2:shared", self.packager.items[0].options)

    def test_auto_managed_subdirectory(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}

"""
        cwd = os.path.join(self.tmp_folder, "subdirectory")
        tools.save(os.path.join(cwd, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(username="lasote", cwd=cwd)
        self.packager.add_common_builds()
        self.assertGreater(len(self.packager.items), 0)
        self.assertIn("lib:shared", self.packager.items[0].options)

    def test_exported_files(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os"
    exports = "*"
    exports_sources = "source*"

"""
        ci_manager = MockCIManager()
        self.save_conanfile(conanfile)
        tools.save(os.path.join(self.tmp_folder, "other_file"), "Dummy contents")
        tools.save(os.path.join(self.tmp_folder, "source.cpp"), "Dummy contents")
        self.packager = ConanMultiPackager(username="lasote", reference="lib/1.0", ci_manager=ci_manager)
        self.packager.add({}, {}, {}, {})
        self.packager.run()

        ref = ConanFileReference.loads("lib/1.0@lasote/testing")
        pf = self.client_cache.package_layout(ref).export()
        found_in_export = False
        for exported in os.listdir(pf):
            if "other_file" == exported:
                found_in_export = True
                break

        self.assertTrue(found_in_export)

        pf = self.client_cache.package_layout(ref).export_sources()
        found_in_export_sources = False
        for exported in os.listdir(pf):
            if "source.cpp" == exported:
                found_in_export_sources = True
                break

        self.assertTrue(found_in_export_sources)

    def test_build_policy(self):
        ci_manager = MockCIManager()
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    name = "lib"
    version = "1.2"
    settings = "os", "compiler", "build_type", "arch"

"""
        self.save_conanfile(conanfile)
        with tools.environment_append({"CONAN_USERNAME": "lasote"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["17"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               build_policy="outdated",
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

        with tools.environment_append({"CONAN_USERNAME": "lasote",
                                       "CONAN_BUILD_POLICY": "outdated"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["17"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

    def test_multiple_build_policy(self):
        ci_manager = MockCIManager()
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    name = "lib"
    version = "1.2"
    settings = "os", "compiler", "build_type", "arch"

"""
        self.save_conanfile(conanfile)
        with tools.environment_append({"CONAN_USERNAME": "lasote"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["17"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               build_policy=["cascade", "outdated"],
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

        with tools.environment_append({"CONAN_USERNAME": "lasote",
                                       "CONAN_BUILD_POLICY": "outdated, lib"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["17"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

    def test_custom_conanfile(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "lib"
    version = "1.2"
    settings = "os", "compiler", "build_type", "arch"
"""
        tools.save(os.path.join(self.tmp_folder, "foobar.py"), conanfile)
        with tools.environment_append({"CONAN_CONANFILE": "foobar.py"}):
            self.packager = ConanMultiPackager(username="pepe",
                                               channel="mychannel",
                                               out=self.output.write)
            self.packager.add({}, {}, {}, {})
            self.packager.run()
        self.assertIn("conanfile                 | foobar.py", self.output)

        tools.save(os.path.join(self.tmp_folder, "custom_recipe.py"), conanfile)
        self.packager = ConanMultiPackager(username="pepe",
                                           channel="mychannel",
                                           conanfile="custom_recipe.py",
                                           out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run()
        self.assertIn("conanfile                 | custom_recipe.py", self.output)

    def test_partial_reference(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foobar"
    version = "0.1.0"

    def configure(self):
        self.output.info("hello all")
"""
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        with tools.environment_append({"CONAN_REFERENCE": "foobar/0.1.0@"}):
            self.packager = ConanMultiPackager(out=self.output.write)
            self.packager.add({}, {}, {}, {})
            self.packager.run()
        self.assertIn("partial_reference         | foobar/0.1.0@", self.output)

        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run()
        self.assertIn("partial_reference         | foobar/0.1.0@", self.output)

    def test_save_packages_summary(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foobar"
    version = "0.1.0"

    def configure(self):
        self.output.info("hello all")
"""
        json_file = 'cpt_summary_file.json'
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run(summary_file=json_file)
        self.assertTrue(os.path.isfile(json_file))
        with open(json_file) as json_content:
            json_data = json.load(json_content)
            self.assertFalse(json_data[0]["package"]["error"])

        json_file = "_" + json_file
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run()
        self.packager.save_packages_summary(json_file)
        self.assertTrue(os.path.isfile(json_file))
        with open(json_file) as json_content:
            json_data = json.load(json_content)
            self.assertFalse(json_data[0]["package"]["error"])

        json_file = "__" + json_file
        with tools.environment_append({"CPT_SUMMARY_FILE": json_file}):
            self.packager = ConanMultiPackager(out=self.output.write)
            self.packager.add({}, {}, {}, {})
            self.packager.run()
            self.assertTrue(os.path.isfile(json_file))
            with open(json_file) as json_content:
                json_data = json.load(json_content)
                self.assertFalse(json_data[0]["package"]["error"])

    def test_disable_test_folder(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
"""
        self.save_conanfile(conanfile)
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    def test(self):
        raise Exception("Should not run")
"""
        tools.save(os.path.join(self.tmp_folder, "test_package", "conanfile.py"), conanfile)
        with tools.environment_append({"CPT_TEST_FOLDER": "False"}):
            self.packager = ConanMultiPackager(out=self.output.write)
            self.packager.add_common_builds()
            self.packager.run()

    def test_invalid_test_folder(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
"""
        self.save_conanfile(conanfile)
        for test_folder in ["True", "foobar"]:
            with tools.environment_append({"CPT_TEST_FOLDER": test_folder}):
                self.packager = ConanMultiPackager(out=self.output.write)
                self.packager.add_common_builds()
                with self.assertRaises(ConanException) as raised:
                    self.packager.run()
                    self.assertIn("test folder '{}' not available, or it doesn't have a conanfile.py"
                                  .format(test_folder),
                                  str(raised.exception))

    def test_custom_name_version(self):
        conanfile = """from conans import ConanFile
from datetime import date
class Pkg(ConanFile):

    def configure(self):
        self.output.info("hello all")

    def set_name(self):
        self.name = "foobar"

    def set_version(self):
        today = date.today()
        self.version = today.strftime("%Y%B%d")
"""
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add_common_builds(pure_c=False)
        self.packager.run()

    def test_header_only_option_true(self):
        header_only = self._test_header_only(False)
        self.assertEqual(header_only, 1)
        self.packager.run()

    def test_header_only_option_false(self):
        header_only = self._test_header_only(True)
        self.assertEqual(header_only, int(len(self.packager.builds) / 2))
        self.packager.run()

    def _test_header_only(self, default_value):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "qux"
    version = "0.1.0"
    settings = "os"
    options = {"header_only": [True, False], "shared": [True, False], "fPIC": [True, False]}
    default_options = {"header_only": %s, "shared": False, "fPIC": True}

    def configure(self):
        if self.options.header_only:
            del self.options.shared
            del self.options.fPIC

    def package_id(self):
        if self.options.header_only:
            self.info.header_only()
""" % default_value
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add_common_builds(pure_c=False)

        header_only = 0
        for build in self.packager.builds:
            _, options, _, _ = build
            if options.get("qux:header_only") == (not default_value):
                header_only += 1
        return header_only

    def test_build_all_option_values(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "qux"
    version = "0.1.0"
    options = {"shared": [True, False], "fPIC": [True, False],
               "header_only": [True, False], "foo": [True, False],
               "bar": ["baz", "qux", "foobar"], "blah": "ANY"}
    default_options = {"shared": False, "fPIC": True, "header_only": False,
                       "foo": False, "bar": "baz", "blah": 42}

    def configure(self):
        self.output.info("hello all")
"""
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add_common_builds(pure_c=False, build_all_options_values=["qux:foo", "qux:bar", "qux:blah"])
        self.packager.run()



from cpt.printer import Printer
from cpt.remotes import RemotesManager
from cpt.test.integration.base import BaseTest


class RemotesTest(BaseTest):

    def test_duplicated_remotes_with_different_url(self):

        self.api.remote_remove("conancenter")
        self.api.remote_add("upload_repo", "url_different", True)

        manager = RemotesManager(self.api, Printer(), remotes_input="url1@True@upload_repo",
                                 upload_input="url1@True@upload_repo")

        remotes = self.api.remote_list()
        self.assertIsNotNone(manager._get_remote_by_name(remotes, "upload_repo"))

        manager.add_remotes_to_conan()

        self.assertEquals(len(self.api.remote_list()), 1)

        manager.add_remotes_to_conan()

        self.assertEquals(len(self.api.remote_list()), 1)

    def test_duplicated_remotes_with_same_url(self):

        self.api.remote_remove("conancenter")
        self.api.remote_add("upload_repo", "url1", True)

        manager = RemotesManager(self.api, Printer(), remotes_input="url1@True@upload_repo",
                                 upload_input="url1@True@upload_repo")

        remotes = self.api.remote_list()
        self.assertIsNotNone(manager._get_remote_by_name(remotes, "upload_repo"))

        manager.add_remotes_to_conan()
        self.assertEquals(len(self.api.remote_list()), 1)



import os
import platform
import unittest
import sys

from collections import defaultdict

from cpt.builds_generator import BuildConf
from cpt.packager import ConanMultiPackager
from conans import tools
from cpt.test.utils.tools import TestBufferConanOutput
from conans.model.ref import ConanFileReference
from cpt.test.unit.utils import MockConanAPI, MockRunner, MockCIManager


def platform_mock_for(so):
    class PlatformInfoMock(object):
        def system(self):
            return so
    return PlatformInfoMock()


class AppTest(unittest.TestCase):

    def setUp(self):
        self.runner = MockRunner()
        self.conan_api = MockConanAPI()
        self.ci_manager = MockCIManager()
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           reference="lib/1.0",
                                           ci_manager=self.ci_manager)

        for provider in ["APPVEYOR", "TRAVIS", "GITHUB_ACTIONS"]:
            if provider in os.environ:
                del os.environ[provider]

    def _add_build(self, number, compiler=None, version=None):
        self.packager.add({"os": "os%d" % number, "compiler": compiler or "compiler%d" % number,
                           "compiler.version": version or "4.3"},
                          {"option%d" % number: "value%d" % number,
                           "option%d" % number: "value%d" % number})

    def test_remove_build_if(self):
        self.packager.add({"arch": "x86", "build_type": "Release", "compiler": "gcc", "compiler.version": "6"})
        self.packager.add({"arch": "x86", "build_type": "Debug", "compiler": "gcc", "compiler.version": "6"})
        self.packager.add({"arch": "x86", "build_type": "Release", "compiler": "gcc", "compiler.version": "7"})
        self.packager.add({"arch": "x86", "build_type": "Debug", "compiler": "gcc", "compiler.version": "7"})

        self.packager.remove_build_if(lambda build: build.settings["compiler.version"] == "6")

        packager_expected = ConanMultiPackager("lasote", "mychannel",
                                               runner=self.runner,
                                               conan_api=self.conan_api,
                                               reference="lib/1.0",
                                               ci_manager=self.ci_manager)

        packager_expected.add({"arch": "x86", "build_type": "Release", "compiler": "gcc", "compiler.version": "7"})
        packager_expected.add({"arch": "x86", "build_type": "Debug", "compiler": "gcc", "compiler.version": "7"})

        self.assertEqual([tuple(a) for a in self.packager.items], packager_expected.items)

    def test_update_build_if(self):
        self.packager.add({"os": "Windows"})
        self.packager.add({"os": "Linux"})

        self.packager.update_build_if(lambda build: build.settings["os"] == "Windows",
                                      new_build_requires={"*": ["7zip/19.00"]})

        packager_expected = ConanMultiPackager("lasote", "mychannel",
                                               runner=self.runner,
                                               conan_api=self.conan_api,
                                               reference="lib/1.0",
                                               ci_manager=self.ci_manager)

        packager_expected.add({"os": "Windows"}, {}, {}, {"*": ["7zip/19.00"]})
        packager_expected.add({"os": "Linux"})

        self.assertEqual([tuple(a) for a in self.packager.items], packager_expected.items)

    def test_add_common_builds_update_build_if(self):
        self.packager.add_common_builds()
        self.packager.update_build_if(lambda build: build.settings["build_type"] == "Debug",
                                      new_options={"foo:bar": True})
        self.packager.update_build_if(lambda build: build.settings["build_type"] == "Release",
                                      new_options={"foo:qux": False})

        for settings, options, _, _, _ in self.packager.items:
            if settings["build_type"] == "Release":
                self.assertEqual(options, {"foo:qux": False})
            else:
                self.assertEqual(options, {"foo:bar": True})

    def test_full_profile(self):
        self.packager.add({"os": "Windows", "compiler": "gcc"},
                          {"option1": "One"},
                          {"VAR_1": "ONE",
                           "VAR_2": "TWO"},
                          {"*": ["myreference/1.0@lasote/testing"]})
        self.packager.run_builds(1, 1)
        profile = self.conan_api.get_profile_from_call_index(1)
        self.assertEquals(profile.settings["os"], "Windows")
        self.assertEquals(profile.settings["compiler"], "gcc")
        self.assertEquals(profile.options.as_list(), [("option1", "One")])
        self.assertEquals(profile.env_values.data[None]["VAR_1"], "ONE")
        self.assertEquals(profile.env_values.data[None]["VAR_2"], "TWO")
        self.assertEquals(profile.build_requires["*"],
                          [ConanFileReference.loads("myreference/1.0@lasote/testing")])

    def test_profile_environ(self):
        self.packager.add({"os": "Windows", "compiler": "gcc"},
                          {"option1": "One"},
                          {"VAR_1": "ONE",
                           "VAR_2": "TWO"},
                          {"*": ["myreference/1.0@lasote/testing"]})
        with tools.environment_append({"CONAN_BUILD_REQUIRES": "br1/1.0@conan/testing"}):
            self.packager.run_builds(1, 1)
            profile = self.conan_api.get_profile_from_call_index(1)
            self.assertEquals(profile.build_requires["*"],
                              [ConanFileReference.loads("myreference/1.0@lasote/testing"),
                               ConanFileReference.loads("br1/1.0@conan/testing")])

    def test_pages(self):
        for number in range(10):
            self._add_build(number)

        # 10 pages, 1 per build
        self.packager.run_builds(1, 10)
        self.conan_api.assert_tests_for([0])

        # 2 pages, 5 per build
        self.conan_api.reset()
        self.packager.run_builds(1, 2)
        self.conan_api.assert_tests_for([0, 2, 4, 6, 8])

        self.conan_api.reset()
        self.packager.run_builds(2, 2)
        self.conan_api.assert_tests_for([1, 3, 5, 7, 9])

        # 3 pages, 4 builds in page 1 and 3 in the rest of pages
        self.conan_api.reset()
        self.packager.run_builds(1, 3)
        self.conan_api.assert_tests_for([0, 3, 6, 9])

        self.conan_api.reset()
        self.packager.run_builds(2, 3)
        self.conan_api.assert_tests_for([1, 4, 7])

        self.conan_api.reset()
        self.packager.run_builds(3, 3)
        self.conan_api.assert_tests_for([2, 5, 8])

    def test_deprecation_gcc(self):

        with self.assertRaisesRegexp(Exception, "DEPRECATED GCC MINOR VERSIONS!"):
            ConanMultiPackager(username="lasote",
                               channel="mychannel",
                               runner=self.runner,
                               conan_api=self.conan_api,
                               gcc_versions=["4.3", "5.4"],
                               use_docker=True,
                               reference="zlib/1.2.11",
                               ci_manager=self.ci_manager)

    def test_32bits_images(self):
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      runner=self.runner,
                                      use_docker=True,
                                      docker_32_images=True,
                                      reference="zlib/1.2.11",
                                      ci_manager=self.ci_manager)

        packager.add({"arch": "x86", "compiler": "gcc", "compiler.version": "6"})
        packager.run_builds(1, 1)
        self.assertIn("docker pull conanio/gcc6-x86", self.runner.calls[0])

        self.runner.reset()
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      use_docker=True,
                                      docker_32_images=False,
                                      reference="zlib/1.2.11",
                                      ci_manager=self.ci_manager)

        packager.add({"arch": "x86", "compiler": "gcc", "compiler.version": "6"})
        packager.run_builds(1, 1)
        self.assertNotIn("docker pull conanio/gcc6-i386", self.runner.calls[0])

        self.runner.reset()
        with tools.environment_append({"CONAN_DOCKER_32_IMAGES": "1"}):
            packager = ConanMultiPackager(username="lasote",
                                          channel="mychannel",
                                          runner=self.runner,
                                          conan_api=self.conan_api,
                                          use_docker=True,
                                          reference="zlib/1.2.11",
                                          ci_manager=self.ci_manager)

            packager.add({"arch": "x86", "compiler": "gcc", "compiler.version": "6"})
            packager.run_builds(1, 1)
            self.assertIn("docker pull conanio/gcc6-x86", self.runner.calls[0])

        self.runner.reset()
        # Test the opossite
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      use_docker=True,
                                      docker_32_images=False,
                                      reference="zlib/1.2.11",
                                      ci_manager=self.ci_manager)

        packager.add({"arch": "x86", "compiler": "gcc", "compiler.version": "6"})
        packager.run_builds(1, 1)
        self.assertIn("docker pull conanio/gcc6", self.runner.calls[0])

    def test_docker_gcc(self):
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           gcc_versions=["4.3", "5"],
                                           use_docker=True,
                                           reference="zlib/1.2.11",
                                           ci_manager=self.ci_manager)
        self._add_build(1, "gcc", "4.3")
        self._add_build(2, "gcc", "4.3")
        self._add_build(3, "gcc", "4.3")

        self.packager.run_builds(1, 2)
        self.assertIn("docker pull conanio/gcc43", self.runner.calls[0])
        self.assertIn('docker run ', self.runner.calls[1])
        self.assertNotIn('sudo pip', self.runner.calls[1])
        self.assertIn('pip install', self.runner.calls[1])
        self.assertIn('os=os1', self.runner.calls[4])
        self.packager.run_builds(1, 2)
        self.assertIn("docker pull conanio/gcc43", self.runner.calls[0])

        # Next build from 4.3 is cached, not pulls are performed
        self.assertIn('os=os3', self.runner.calls[5])

        for the_bool in ["True", "False"]:
            self.runner.reset()
            with tools.environment_append({"CONAN_DOCKER_USE_SUDO": the_bool}):
                self.packager = ConanMultiPackager(username="lasote",
                                                   channel="mychannel",
                                                   runner=self.runner,
                                                   conan_api=self.conan_api,
                                                   gcc_versions=["4.3", "5"],
                                                   use_docker=True,
                                                   reference="zlib/1.2.11",
                                                   ci_manager=self.ci_manager)
                self._add_build(1, "gcc", "4.3")
                self.packager.run_builds(1, 2)
                if the_bool == "True":
                    self.assertIn("sudo -E docker run", self.runner.calls[-1])
                else:
                    self.assertNotIn("sudo -E docker run", self.runner.calls[-1])
                    self.assertIn("docker run", self.runner.calls[-1])
            self.runner.reset()
            with tools.environment_append({"CONAN_PIP_USE_SUDO": the_bool}):
                self.packager = ConanMultiPackager(username="lasote",
                                                   channel="mychannel",
                                                   runner=self.runner,
                                                   conan_api=self.conan_api,
                                                   gcc_versions=["4.3", "5"],
                                                   use_docker=True,
                                                   reference="zlib/1.2.11",
                                                   ci_manager=self.ci_manager)
                self._add_build(1, "gcc", "4.3")
                self.packager.run_builds(1, 2)
                if the_bool == "True":
                    self.assertIn("sudo -E pip", self.runner.calls[1])
                else:
                    self.assertNotIn("sudo -E pip", self.runner.calls[1])
                    self.assertIn("pip", self.runner.calls[1])

    def test_docker_clang(self):
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           clang_versions=["3.8", "4.0"],
                                           use_docker=True,
                                           reference="zlib/1.2.11",
                                           ci_manager=self.ci_manager)

        self._add_build(1, "clang", "3.8")
        self._add_build(2, "clang", "3.8")
        self._add_build(3, "clang", "3.8")

        self.packager.run_builds(1, 2)
        self.assertIn("docker pull conanio/clang38", self.runner.calls[0])
        self.assertIn('docker run ', self.runner.calls[1])
        self.assertIn('os=os1', self.runner.calls[4])

        # Next build from 3.8 is cached, not pulls are performed
        self.assertIn('os=os3', self.runner.calls[5])

    def test_docker_gcc_and_clang(self):
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           gcc_versions=["5", "6"],
                                           clang_versions=["3.9", "4.0"],
                                           use_docker=True,
                                           reference="zlib/1.2.11",
                                           ci_manager=self.ci_manager)

        self._add_build(1, "gcc", "5")
        self._add_build(2, "gcc", "5")
        self._add_build(3, "gcc", "5")
        self._add_build(4, "clang", "3.9")
        self._add_build(5, "clang", "3.9")
        self._add_build(6, "clang", "3.9")

        self.packager.run_builds(1, 2)
        self.assertIn("docker pull conanio/gcc5", self.runner.calls[0])
        self.assertIn('docker run ', self.runner.calls[1])

        self.assertIn('os=os1', self.runner.calls[4])
        self.assertIn('os=os3', self.runner.calls[5])

        self.packager.run_builds(2, 2)
        self.assertIn("docker pull conanio/clang39", self.runner.calls[16])
        self.assertIn('docker run ', self.runner.calls[17])
        self.assertIn('os=os4', self.runner.calls[20])
        self.assertIn('os=os6', self.runner.calls[21])

    def test_upload_false(self):
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      upload=False, reference="zlib/1.2.11",
                                      ci_manager=self.ci_manager)
        self.assertFalse(packager._upload_enabled())

    def test_docker_env_propagated(self):
        # test env
        with tools.environment_append({"CONAN_FAKE_VAR": "32"}):
            self.packager = ConanMultiPackager(username="lasote",
                                               channel="mychannel",
                                               runner=self.runner,
                                               conan_api=self.conan_api,
                                               gcc_versions=["5", "6"],
                                               clang_versions=["3.9", "4.0"],
                                               use_docker=True,
                                               reference="zlib/1.2.11",
                                               ci_manager=self.ci_manager)
            self._add_build(1, "gcc", "5")
            self.packager.run_builds(1, 1)
            self.assertIn('-e CONAN_FAKE_VAR="32"', self.runner.calls[-1])

    def test_docker_home_env(self):
        with tools.environment_append({"CONAN_DOCKER_HOME": "/some/dir"}):
            self.packager = ConanMultiPackager(username="lasote",
                                               channel="mychannel",
                                               runner=self.runner,
                                               conan_api=self.conan_api,
                                               gcc_versions=["5", "6"],
                                               clang_versions=["3.9", "4.0"],
                                               use_docker=True,
                                               reference="zlib/1.2.11",
                                               ci_manager=self.ci_manager)
            self._add_build(1, "gcc", "5")
            self.packager.run_builds(1, 1)
            self.assertIn('-e CONAN_DOCKER_HOME="/some/dir"',
                          self.runner.calls[-1])
            self.assertEquals(self.packager.docker_conan_home, "/some/dir")

    def test_docker_home_opt(self):
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           gcc_versions=["5", "6"],
                                           clang_versions=["3.9", "4.0"],
                                           use_docker=True,
                                           docker_conan_home="/some/dir",
                                           reference="zlib/1.2.11",
                                           ci_manager=self.ci_manager)
        self._add_build(1, "gcc", "5")
        self.packager.run_builds(1, 1)
        self.assertEquals(self.packager.docker_conan_home, "/some/dir")

    def test_docker_invalid(self):
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           use_docker=True,
                                           reference="zlib/1.2.11",
                                           ci_manager=self.ci_manager)

        self._add_build(1, "msvc", "10")

        # Only clang and gcc have docker images
        self.assertRaises(Exception, self.packager.run_builds)

    def test_assign_builds_retrocompatibility(self):
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           gcc_versions=["4.3", "5"],
                                           use_docker=True,
                                           reference="lib/1.0",
                                           ci_manager=self.ci_manager)
        self.packager.add_common_builds()
        self.packager.builds = [({"os": "Windows"}, {"option": "value"})]
        self.assertEquals(self.packager.items, [BuildConf(settings={'os': 'Windows'},
                                                          options={'option': 'value'},
                                                          env_vars={}, build_requires={},
                                                          reference="lib/1.0@lasote/mychannel")])

    def test_only_mingw(self):

        mingw_configurations = [("4.9", "x86_64", "seh", "posix")]
        builder = ConanMultiPackager(mingw_configurations=mingw_configurations, visual_versions=[],
                                     username="Pepe", platform_info=platform_mock_for("Windows"),
                                     reference="lib/1.0", ci_manager=self.ci_manager)
        with tools.environment_append({"CONAN_SHARED_OPTION_NAME": "zlib:shared"}):
            builder.add_common_builds(pure_c=True)
        expected = [({'compiler.exception': 'seh', 'compiler.libcxx': "libstdc++",
                      'compiler.threads': 'posix', 'compiler.version': '4.9', 'arch': 'x86_64',
                      'build_type': 'Release', 'compiler': 'gcc'},
                     {'zlib:shared': False},
                     {},
                     {'*': [ConanFileReference.loads("mingw-w64/8.1")]}),
                    ({'compiler.exception': 'seh', 'compiler.libcxx': "libstdc++", 'arch': 'x86_64',
                      'compiler.threads': 'posix', 'compiler.version': '4.9', 'build_type': 'Debug',
                      'compiler': 'gcc'},
                     {'zlib:shared': False},
                     {},
                     {'*': [ConanFileReference.loads("mingw-w64/8.1")]}),

                    ({'compiler.exception': 'seh', 'compiler.libcxx': "libstdc++",
                      'compiler.threads': 'posix', 'compiler.version': '4.9', 'arch': 'x86_64',
                      'build_type': 'Release', 'compiler': 'gcc'},
                     {'zlib:shared': True},
                     {},
                     {'*': [ConanFileReference.loads("mingw-w64/8.1")]}),
                    ({'compiler.exception': 'seh', 'compiler.libcxx': "libstdc++", 'arch': 'x86_64',
                      'compiler.threads': 'posix', 'compiler.version': '4.9', 'build_type': 'Debug',
                      'compiler': 'gcc'},
                     {'zlib:shared': True},
                     {},
                     {'*': [ConanFileReference.loads("mingw-w64/8.1")]})]

        self.assertEquals([tuple(a) for a in builder.builds], expected)

    def test_named_pages(self):
        builder = ConanMultiPackager(username="Pepe", reference="zlib/1.2.11",
                                     ci_manager=self.ci_manager)
        named_builds = defaultdict(list)
        with tools.environment_append({"CONAN_SHARED_OPTION_NAME": "zlib:shared"}):
            builder.add_common_builds(pure_c=True)
            for settings, options, env_vars, build_requires, _ in builder.items:
                named_builds[settings['arch']].append([settings, options, env_vars, build_requires])
            builder.named_builds = named_builds

        self.assertEquals(builder.builds, [])
        if platform.system() == "Darwin":  # Not default x86 in Macos
            self.assertEquals(len(builder.named_builds), 1)
            self.assertFalse("x86" in builder.named_builds)
            self.assertTrue("x86_64" in builder.named_builds)
        else:
            self.assertEquals(len(builder.named_builds), 2)
            self.assertTrue("x86" in builder.named_builds)
            self.assertTrue("x86_64" in builder.named_builds)

    def test_remotes(self):
        runner = MockRunner()
        builder = ConanMultiPackager(username="Pepe",
                                     remotes=["url1", "url2"],
                                     runner=runner,
                                     conan_api=self.conan_api,
                                     reference="lib/1.0@lasote/mychannel",
                                     ci_manager=self.ci_manager)

        self.assertEquals(self.conan_api.calls[1].args[1], "url1")
        self.assertEquals(self.conan_api.calls[1].kwargs["insert"], -1)
        self.assertEquals(self.conan_api.calls[3].args[1], "url2")
        self.assertEquals(self.conan_api.calls[3].kwargs["insert"], -1)

        runner = MockRunner()
        self.conan_api = MockConanAPI()
        builder = ConanMultiPackager(username="Pepe",
                                     remotes="myurl1",
                                     runner=runner,
                                     conan_api=self.conan_api,
                                     reference="lib/1.0@lasote/mychannel",
                                     ci_manager=self.ci_manager)

        self.assertEquals(self.conan_api.calls[1].args[1], "myurl1")
        self.assertEquals(self.conan_api.calls[1].kwargs["insert"], -1)

        # Named remotes, with SSL flag
        runner = MockRunner()
        self.conan_api = MockConanAPI()
        remotes = [("u1", True, "my_cool_name1"),
                   ("u2", False, "my_cool_name2")]
        builder = ConanMultiPackager(username="Pepe",
                                     remotes=remotes,
                                     runner=runner,
                                     conan_api=self.conan_api,
                                     reference="lib/1.0@lasote/mychannel",
                                     ci_manager=self.ci_manager)

        self.assertEquals(self.conan_api.calls[1].args[0], "my_cool_name1")
        self.assertEquals(self.conan_api.calls[1].args[1], "u1")
        self.assertEquals(self.conan_api.calls[1].kwargs["insert"], -1)
        self.assertEquals(self.conan_api.calls[1].kwargs["verify_ssl"], True)

        self.assertEquals(self.conan_api.calls[3].args[0], "my_cool_name2")
        self.assertEquals(self.conan_api.calls[3].args[1], "u2")
        self.assertEquals(self.conan_api.calls[3].kwargs["insert"], -1)
        self.assertEquals(self.conan_api.calls[3].kwargs["verify_ssl"], False)

    def test_visual_defaults(self):

        with tools.environment_append({"CONAN_VISUAL_VERSIONS": "10"}):
            builder = ConanMultiPackager(username="Pepe",
                                         platform_info=platform_mock_for("Windows"),
                                         reference="lib/1.0@lasote/mychannel",
                                         ci_manager=self.ci_manager)
            builder.add_common_builds()
            for settings, _, _, _, _ in builder.items:
                self.assertEquals(settings["compiler"], "Visual Studio")
                self.assertEquals(settings["compiler.version"], "10")

        with tools.environment_append({"CONAN_VISUAL_VERSIONS": "10",
                                       "MINGW_CONFIGURATIONS": "4.9@x86_64@seh@posix"}):

            builder = ConanMultiPackager(username="Pepe",
                                         platform_info=platform_mock_for("Windows"),
                                         reference="lib/1.0@lasote/mychannel",
                                         ci_manager=self.ci_manager)
            builder.add_common_builds()
            for settings, _, _, _, _ in builder.items:
                self.assertEquals(settings["compiler"], "gcc")
                self.assertEquals(settings["compiler.version"], "4.9")

    def test_msvc_defaults(self):

        with tools.environment_append({"CONAN_MSVC_VERSIONS": "193",
                                       "CONAN_VISUAL_VERSIONS": ""}):
            builder = ConanMultiPackager(username="Pepe",
                                         platform_info=platform_mock_for("Windows"),
                                         reference="lib/1.0@lasote/mychannel",
                                         ci_manager=self.ci_manager)
            builder.add_common_builds()
            for settings, _, _, _, _ in builder.items:
                self.assertEquals(settings["compiler"], "msvc")
                self.assertEquals(settings["compiler.version"], "193")

        with tools.environment_append({"CONAN_MSVC_VERSIONS": "193",
                                       "CONAN_VISUAL_VERSIONS": "",
                                       "MINGW_CONFIGURATIONS": "4.9@x86_64@seh@posix"}):

            builder = ConanMultiPackager(username="Pepe",
                                         platform_info=platform_mock_for("Windows"),
                                         reference="lib/1.0@lasote/mychannel",
                                         ci_manager=self.ci_manager)
            builder.add_common_builds()
            for settings, _, _, _, _ in builder.items:
                self.assertEquals(settings["compiler"], "gcc")
                self.assertEquals(settings["compiler.version"], "4.9")

    def test_multiple_references(self):
        with tools.environment_append({"CONAN_REFERENCE": "zlib/1.2.8"}):
            builder = ConanMultiPackager(username="Pepe", ci_manager=self.ci_manager)
            builder.add_common_builds(reference="lib/1.0@lasote/mychannel")
            for _, _, _, _, reference in builder.items:
                self.assertEquals(str(reference), "lib/1.0@lasote/mychannel")
            builder.add_common_builds(reference="lib/2.0@lasote/mychannel")
            for _, _, _, _, reference in builder.items:
                self.assertTrue(str(reference) in ("lib/1.0@lasote/mychannel", "lib/2.0@lasote/mychannel"))

    def test_select_defaults_test(self):
        with tools.environment_append({"CONAN_REFERENCE": "zlib/1.2.8"}):
            builder = ConanMultiPackager(platform_info=platform_mock_for("Linux"),
                                         gcc_versions=["4.8", "5"],
                                         username="foo",
                                         reference="lib/1.0@lasote/mychannel",
                                         ci_manager=self.ci_manager)

            self.assertEquals(builder.build_generator._clang_versions, [])

        with tools.environment_append({"CONAN_GCC_VERSIONS": "4.8, 5",
                                       "CONAN_REFERENCE": "zlib/1.2.8"}):
            builder = ConanMultiPackager(platform_info=platform_mock_for("Linux"),
                                         username="foo",
                                         reference="lib/1.0@lasote/mychannel",
                                         ci_manager=self.ci_manager)

            self.assertEquals(builder.build_generator._clang_versions, [])
            self.assertEquals(builder.build_generator._gcc_versions, ["4.8", "5"])

        builder = ConanMultiPackager(platform_info=platform_mock_for("Linux"),
                                     clang_versions=["4.8", "5"],
                                     username="foo",
                                     reference="lib/1.0",
                                     ci_manager=self.ci_manager)

        self.assertEquals(builder.build_generator._gcc_versions, [])

        with tools.environment_append({"CONAN_CLANG_VERSIONS": "4.8, 5",
                                       "CONAN_APPLE_CLANG_VERSIONS": " "}):
            builder = ConanMultiPackager(platform_info=platform_mock_for("Linux"),
                                         username="foo",
                                         reference="lib/1.0",
                                         ci_manager=self.ci_manager)

            self.assertEquals(builder.build_generator._gcc_versions, [])
            self.assertEquals(builder.build_generator._clang_versions, ["4.8", "5"])
            self.assertEquals(builder.build_generator._clang_versions, ["4.8", "5"])
            self.assertEquals(builder.build_generator._apple_clang_versions, [])

    def test_upload(self):
        runner = MockRunner()
        runner.output = "arepo: myurl"
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     upload="myurl", visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=runner,
                                     conan_api=self.conan_api,
                                     remotes="myurl, otherurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()

        # Duplicated upload remote puts upload repo first (in the remotes order)
        self.assertEqual(self.conan_api.calls[1].args[0], 'upload_repo')
        self.assertEqual(self.conan_api.calls[3].args[0], 'remote1')

        # Now check that the upload remote order is preserved if we specify it in the remotes
        runner = MockRunner()
        self.conan_api = MockConanAPI()
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     upload="myurl", visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=runner,
                                     conan_api=self.conan_api,
                                     remotes="otherurl, myurl, moreurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()

        self.assertEqual(self.conan_api.calls[1].args[0], 'remote0')
        self.assertEqual(self.conan_api.calls[3].args[0], 'upload_repo')
        self.assertEqual(self.conan_api.calls[5].args[0], 'remote2')

        runner = MockRunner()
        self.conan_api = MockConanAPI()
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     upload="myurl", visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=runner,
                                     conan_api=self.conan_api,
                                     remotes="otherurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()

        self.assertEqual(self.conan_api.calls[1].args[0], 'remote0')
        self.assertEqual(self.conan_api.calls[3].args[0], 'upload_repo')

    def test_build_policy(self):
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     remotes="otherurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     build_policy="outdated",
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()
        self.assertEquals(["outdated"], self.conan_api.calls[-1].kwargs["build_modes"])

    def test_multiple_build_policy(self):
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     remotes="otherurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     build_policy=["Hello", "outdated"],
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()
        self.assertEquals(["Hello", "outdated"], self.conan_api.calls[-1].kwargs["build_modes"])


        for build_policy, expected in [("missing", ["missing"]), ("all",[]), ("Hello,missing", ["Hello", "missing"])]:
            with tools.environment_append({"CONAN_BUILD_POLICY": build_policy}):
                self.conan_api = MockConanAPI()
                builder = ConanMultiPackager(username="pepe", channel="testing",
                                             reference="Hello/0.1", password="password",
                                             visual_versions=[], gcc_versions=[],
                                             apple_clang_versions=[],
                                             runner=self.runner,
                                             conan_api=self.conan_api,
                                             remotes="otherurl",
                                             platform_info=platform_mock_for("Darwin"),
                                             build_policy=build_policy,
                                             ci_manager=self.ci_manager)
                builder.add_common_builds()
                builder.run()
                self.assertEquals(expected, self.conan_api.calls[-1].kwargs["build_modes"])

    def test_test_folder(self):
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     remotes="otherurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     test_folder="foobar",
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()
        self.assertEquals("foobar", self.conan_api.calls[-1].kwargs["test_folder"])

        with tools.environment_append({"CONAN_BUILD_POLICY": "missing"}):
            self.conan_api = MockConanAPI()
            builder = ConanMultiPackager(username="pepe", channel="testing",
                                         reference="Hello/0.1", password="password",
                                         visual_versions=[], gcc_versions=[],
                                         apple_clang_versions=[],
                                         runner=self.runner,
                                         conan_api=self.conan_api,
                                         remotes="otherurl",
                                         platform_info=platform_mock_for("Darwin"),
                                         build_policy=None,
                                         ci_manager=self.ci_manager)
            builder.add_common_builds()
            builder.run()
            self.assertEquals(None, self.conan_api.calls[-1].kwargs["test_folder"])

    def test_check_credentials(self):

        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     upload="myurl", visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     platform_info=platform_mock_for("Darwin"),
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()

        # When activated, check credentials before to create the profiles
        self.assertEqual(self.conan_api.calls[2].name, 'authenticate')
        self.assertEqual(self.conan_api.calls[3].name, 'create_profile')

        self.conan_api = MockConanAPI()
        # If we skip the credentials check, the login will be performed just before the upload
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     upload="myurl", visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     platform_info=platform_mock_for("Darwin"),
                                     skip_check_credentials=True,
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()
        self.assertNotEqual(self.conan_api.calls[2].name, 'authenticate')

        # No upload, no authenticate
        self.conan_api = MockConanAPI()
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     upload=None, visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     platform_info=platform_mock_for("Darwin"),
                                     skip_check_credentials=True,
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()
        for action in self.conan_api.calls:
            self.assertNotEqual(action.name, 'authenticate')
            self.assertNotEqual(action.name, 'upload')

    def channel_detector_test(self):

        for branch, expected_channel in [("testing", "a_channel"),
                                         ("dummy", "a_channel"),
                                         ("stabl", "a_channel"),
                                         ("stabl/something", "a_channel"),
                                         ("stable", "stable"),
                                         ("stable/something", "stable"),
                                         ("releas", "a_channel"),
                                         ("releas/something", "a_channel"),
                                         ("release", "stable"),
                                         ("release/something", "stable"),
                                         ("maste", "a_channel"),
                                         ("maste/something", "a_channel"),
                                         ("master", "stable"),
                                         ("main", "stable"),
                                         ("masterSomething", "a_channel"),
                                         ("master/something", "a_channel")]:
            builder = ConanMultiPackager(username="pepe",
                                         channel="a_channel",
                                         reference="lib/1.0",
                                         ci_manager=MockCIManager(current_branch=branch))

            self.assertEquals(builder.channel, expected_channel, "Not match for branch %s" % branch)

    def channel_detector_test_custom_branch_patterns(self):

        for branch, expected_channel in [("trunk", "stable"),
                                         ("trunk/something", "a_channel"),
                                         ("tags/something", "stable"),
                                         ("tagsSomething", "a_channel"),
                                         ("stable", "a_channel"),
                                         ("stable/something", "a_channel"),
                                         ("release", "a_channel"),
                                         ("release/something", "a_channel"),
                                         ("master", "a_channel")]:
            # test env var
            with tools.environment_append({"CONAN_STABLE_BRANCH_PATTERN": "trunk$ tags/.*"}):
                builder = ConanMultiPackager(username="pepe",
                                             channel="a_channel",
                                             reference="lib/1.0",
                                             ci_manager=MockCIManager(current_branch=branch))

                self.assertEquals(builder.channel, expected_channel, "Not match for branch %s" % branch)
            # test passing as argument
            builder = ConanMultiPackager(username="pepe",
                                         channel="a_channel",
                                         reference="lib/1.0",
                                         stable_branch_pattern="trunk$ tags/.*",
                                         ci_manager=MockCIManager(current_branch=branch))

            self.assertEquals(builder.channel, expected_channel, "Not match for branch %s" % branch)

    def test_pip_conanio_image(self):
        self.packager = ConanMultiPackager(username="lasote",
                                            channel="mychannel",
                                            runner=self.runner,
                                            conan_api=self.conan_api,
                                            gcc_versions=["4.3", "5"],
                                            use_docker=True,
                                            docker_image='conanio/gcc43',
                                            reference="zlib/1.2.11",
                                            ci_manager=self.ci_manager)
        self._add_build(1, "gcc", "4.3")
        self.packager.run_builds(1, 2)
        self.assertNotIn("sudo -E pip", self.runner.calls[1])
        self.assertIn("pip", self.runner.calls[1])

        self.runner.reset()
        self.packager = ConanMultiPackager(username="lasote",
                                            channel="mychannel",
                                            runner=self.runner,
                                            conan_api=self.conan_api,
                                            gcc_versions=["4.3", "5"],
                                            docker_image='conanio/gcc43',
                                            reference="zlib/1.2.11",
                                            ci_manager=self.ci_manager)
        self._add_build(1, "gcc", "4.3")
        self.packager.run_builds(1, 2)
        self.assertNotIn("sudo -E pip", self.runner.calls[1])
        self.assertIn("pip", self.runner.calls[1])


    @unittest.skipIf(sys.platform.startswith("win"), "Requires Linux")
    def test_pip_docker_sudo(self):
        self.packager = ConanMultiPackager(username="lasote",
                                            channel="mychannel",
                                            runner=self.runner,
                                            conan_api=self.conan_api,
                                            gcc_versions=["4.3", "5"],
                                            docker_image='foobar/gcc43',
                                            reference="zlib/1.2.11",
                                            ci_manager=self.ci_manager)
        self._add_build(1, "gcc", "4.3")
        self.packager.run_builds(1, 2)
        self.assertIn("sudo -E pip", self.runner.calls[1])

        self.runner.reset()
        with tools.environment_append({"CONAN_PIP_USE_SUDO": "True"}):
            self.packager = ConanMultiPackager(username="lasote",
                                                channel="mychannel",
                                                runner=self.runner,
                                                conan_api=self.conan_api,
                                                gcc_versions=["4.3", "5"],
                                                docker_image='conanio/gcc43',
                                                reference="zlib/1.2.11",
                                                ci_manager=self.ci_manager)
        self._add_build(1, "gcc", "4.3")
        self.packager.run_builds(1, 2)
        self.assertIn("sudo -E pip", self.runner.calls[1])

    def test_regular_pip_command(self):
        """ CPT Should call `pip` when CONAN_PIP_PACKAGE or CONAN_PIP_INSTALL are declared.
        """
        with tools.environment_append({"CONAN_USERNAME": "foobar",
                                       "CONAN_PIP_PACKAGE": "conan==1.0.0-dev",
                                       "CONAN_PIP_INSTALL": "foobar==0.1.0"}):
            output = TestBufferConanOutput()
            self.packager = ConanMultiPackager(username="lasote",
                                               channel="mychannel",
                                               reference="lib/1.0",
                                               ci_manager=self.ci_manager,
                                               out=output.write,
                                               conan_api=self.conan_api,
                                               runner=self.runner,
                                               exclude_vcvars_precommand=True)
            self.packager.add_common_builds()
            self.packager.run()
            self.assertIn("[pip_update]", output)
            self.assertIn(" pip install -q conan==1.0.0-dev", self.runner.calls)
            self.assertIn(" pip install -q foobar==0.1.0", self.runner.calls)

    def test_custom_pip_command(self):
        """ CPT should run custom `pip` path when CONAN_PIP_COMMAND is declared.
        """
        pip = "pip3" if tools.which("pip3") else "pip2"
        with tools.environment_append({"CONAN_USERNAME": "foobar",
                                       "CONAN_PIP_PACKAGE": "conan==0.1.0",
                                       "CONAN_PIP_INSTALL": "foobar==0.1.0",
                                       "CONAN_PIP_COMMAND": pip}):
            output = TestBufferConanOutput()
            self.packager = ConanMultiPackager(username="lasote",
                                               channel="mychannel",
                                               reference="lib/1.0",
                                               ci_manager=self.ci_manager,
                                               out=output.write,
                                               conan_api=self.conan_api,
                                               runner=self.runner,
                                               exclude_vcvars_precommand=True)
            self.packager.add_common_builds()
            self.packager.run()
            self.assertIn("[pip_update]", output)
            self.assertIn(" {} install -q conan==0.1.0".format(pip), self.runner.calls)
            self.assertIn(" {} install -q foobar==0.1.0".format(pip), self.runner.calls)

    def test_invalid_pip_command(self):
        """ CPT should not accept invalid `pip` command when CONAN_PIP_COMMAND is declared.
        """
        with tools.environment_append({"CONAN_USERNAME": "foobar",
                                       "CONAN_PIP_PACKAGE": "conan==0.1.0",
                                       "CONAN_PIP_COMMAND": "/bin/bash"}):
            output = TestBufferConanOutput()
            with self.assertRaises(Exception) as context:
                self.packager = ConanMultiPackager(username="lasote",
                                                channel="mychannel",
                                                reference="lib/1.0",
                                                ci_manager=self.ci_manager,
                                                out=output.write,
                                                conan_api=self.conan_api,
                                                runner=self.runner,
                                                exclude_vcvars_precommand=True)
                self.packager.add_common_builds()
                self.packager.run()

                self.assertTrue("CONAN_PIP_COMMAND: '/bin/bash' is not a valid pip command" in context.exception)
            self.assertNotIn("[pip_update]", output)

    def test_skip_recipe_export(self):

        def _check_create_calls(skip_recipe_export):
            not_export = "not_export"
            creates = self.conan_api.get_creates()
            if skip_recipe_export:
                # Only first call should export recipe
                self.assertFalse(self.assertFalse(creates[0].kwargs[not_export]))
                for call in creates[1:]:
                    self.assertTrue(call.kwargs[not_export])
            else:
                for call in creates:
                    self.assertFalse(call.kwargs[not_export])

        output = TestBufferConanOutput()
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      visual_versions=["17"],
                                      archs=["x86", "x86_64"],
                                      build_types=["Release"],
                                      reference="zlib/1.2.11",
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      ci_manager=self.ci_manager,
                                      out=output.write)
        packager.add_common_builds()
        packager.run()
        _check_create_calls(False)

        with tools.environment_append({"CONAN_SKIP_RECIPE_EXPORT": "True"}):
            self.conan_api.reset()
            packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      visual_versions=["17"],
                                      archs=["x86", "x86_64"],
                                      build_types=["Release"],
                                      reference="zlib/1.2.11",
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      ci_manager=self.ci_manager,
                                      out=output.write)

            packager.add_common_builds()
            packager.run()
            _check_create_calls(True)

        self.conan_api.reset()
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      visual_versions=["17"],
                                      archs=["x86", "x86_64"],
                                      build_types=["Release"],
                                      reference="zlib/1.2.11",
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      ci_manager=self.ci_manager,
                                      skip_recipe_export=True,
                                      out=output.write)
        packager.add_common_builds()
        packager.run()
        _check_create_calls(True)

    def test_skip_recipe_export_docker(self):

        def _check_run_calls(skip_recipe_export):
            env_var = '-e CPT_SKIP_RECIPE_EXPORT="True"'
            run_calls = [call for call in self.runner.calls if "docker run --rm" in call]
            if skip_recipe_export:
                # Only first call should export recipe
                self.assertNotIn(env_var, run_calls[0])
                for call in run_calls[1:]:
                    self.assertIn(env_var, call)
            else:
                for call in run_calls:
                    self.assertNotIn(env_var, call)

        output = TestBufferConanOutput()
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      gcc_versions=["9"],
                                      archs=["x86", "x86_64"],
                                      build_types=["Release"],
                                      reference="zlib/1.2.11",
                                      use_docker=True,
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      ci_manager=self.ci_manager,
                                      out=output.write)
        packager.add_common_builds()
        packager.run()
        _check_run_calls(False)

        with tools.environment_append({"CONAN_SKIP_RECIPE_EXPORT": "True"}):
            self.runner.reset()
            packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      gcc_versions=["9"],
                                      archs=["x86", "x86_64"],
                                      build_types=["Release"],
                                      reference="zlib/1.2.11",
                                      use_docker=True,
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      ci_manager=self.ci_manager,
                                      out=output.write)

            packager.add_common_builds()
            packager.run()
            _check_run_calls(True)

        self.runner.reset()
        packager = ConanMultiPackager(username="lasote",
                                      channel="mychannel",
                                      gcc_versions=["9"],
                                      archs=["x86", "x86_64"],
                                      build_types=["Release"],
                                      reference="zlib/1.2.11",
                                      use_docker=True,
                                      runner=self.runner,
                                      conan_api=self.conan_api,
                                      ci_manager=self.ci_manager,
                                      skip_recipe_export=True,
                                      out=output.write)
        packager.add_common_builds()
        packager.run()
        _check_run_calls(True)

    def test_lockfile(self):
        builder = ConanMultiPackager(username="pepe", channel="testing",
                                     reference="Hello/0.1", password="password",
                                     visual_versions=[], gcc_versions=[],
                                     apple_clang_versions=[],
                                     runner=self.runner,
                                     conan_api=self.conan_api,
                                     remotes="otherurl",
                                     platform_info=platform_mock_for("Darwin"),
                                     lockfile="foobar.lock",
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        builder.run()
        self.assertEquals("foobar.lock", self.conan_api.calls[-1].kwargs["lockfile"])

        with tools.environment_append({"CONAN_LOCKFILE": "couse.lock"}):
            self.conan_api = MockConanAPI()
            builder = ConanMultiPackager(username="pepe", channel="testing",
                                         reference="Hello/0.1", password="password",
                                         visual_versions=[], gcc_versions=[],
                                         apple_clang_versions=[],
                                         runner=self.runner,
                                         conan_api=self.conan_api,
                                         remotes="otherurl",
                                         platform_info=platform_mock_for("Darwin"),
                                         build_policy=None,
                                         ci_manager=self.ci_manager)
            builder.add_common_builds()
            builder.run()
            self.assertEquals("couse.lock", self.conan_api.calls[-1].kwargs["lockfile"])

    def test_pure_c_env_var(self):

        builder = ConanMultiPackager(gcc_versions=["8"], archs=["x86_64"], build_types=["Release"],
                                     username="Pepe", channel="testing",
                                     reference="Hello/0.1",
                                     platform_info=platform_mock_for("Linux"),
                                     ci_manager=self.ci_manager)
        builder.add_common_builds()
        expected = [({'arch': 'x86_64', 'build_type': 'Release',
                      'compiler': 'gcc',
                      'compiler.version': '8'},
                     {},
                     {},
                     {})]
        self.assertEquals([tuple(a) for a in builder.builds], expected)

        builder.builds = []
        with tools.environment_append({"CONAN_PURE_C": "True"}):
            builder.add_common_builds()
        expected = [({'arch': 'x86_64', 'build_type': 'Release',
                      'compiler': 'gcc',
                      'compiler.version': '8'},
                     {},
                     {},
                     {})]
        self.assertEquals([tuple(a) for a in builder.builds], expected)

        builder.builds = []
        with tools.environment_append({"CONAN_PURE_C": "False"}):
            builder.add_common_builds()
        expected = [({'arch': 'x86_64', 'build_type': 'Release',
                      'compiler': 'gcc',
                      'compiler.version': '8',
                      'compiler.libcxx': "libstdc++"},
                     {},
                     {},
                     {}),
                    ({'arch': 'x86_64', 'build_type': 'Release',
                      'compiler': 'gcc',
                      'compiler.version': '8',
                      'compiler.libcxx': "libstdc++11"},
                     {},
                     {},
                     {})]
        self.assertEquals([tuple(a) for a in builder.builds], expected)

    def test_docker_cwd(self):
        cwd = os.path.join(os.getcwd(), 'subdir')
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           runner=self.runner,
                                           conan_api=self.conan_api,
                                           gcc_versions=["9"],
                                           use_docker=True,
                                           reference="zlib/1.2.11",
                                           ci_manager=self.ci_manager,
                                           cwd=cwd)

        self._add_build(1, "gcc", "9")
        self.packager.run_builds(1, 1)
        self.assertIn('docker run --rm -v "%s:%s/project"' % (cwd, self.packager.docker_conan_home), self.runner.calls[4])



import unittest
import os

from cpt.test.utils.tools import TestBufferConanOutput
from cpt.packager import ConanMultiPackager
from cpt.ci_manager import CIManager
from conans import tools
from cpt.printer import Printer


class CIManagerTest(unittest.TestCase):

    def setUp(self):
        # Clean env first
        new_env = {}
        for var, value in os.environ.items():
            if "travis" not in var.lower() and \
               "appveyor" not in var.lower() and \
               "bamboo" not in var.lower():
                new_env[var] = value

        os.environ = new_env
        self.output = TestBufferConanOutput()
        self.printer = Printer(self.output.write)

    # def test_skip(self):
    #     with tools.environment_append({"TRAVIS": "1",
    #                                    "TRAVIS_COMMIT_MESSAGE": "[skip ci]",
    #                                    "TRAVIS_BRANCH": "mybranch"
    #                                    }):
    #         packager = ConanMultiPackager(username="dori", reference="lib/1.0")
    #         # Constructor skipped
    #         ret = packager.run()
    #         self.assertEquals(ret, 99)

    # def test_bamboo_instance(self):
    #     with tools.environment_append({"bamboo_buildNumber": "xx",
    #                                    "bamboo_planRepository_branch": "mybranch"}):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertIsNotNone(manager.get_commit_msg())

    # def test_travis_instance(self):
    #     with tools.environment_append({"TRAVIS": "1",
    #                                    "TRAVIS_COMMIT_MESSAGE": "msg",
    #                                    "TRAVIS_BRANCH": "mybranch",
    #                                    "TRAVIS_COMMIT": "506c89117650bb12252db26d35b8c2385411f175"
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertEquals(manager.get_commit_msg(), "msg")
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")

    # def test_appveyor_instance(self):
    #     with tools.environment_append({"APPVEYOR": "1",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE": "msg",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE_EXTENDED": "more",
    #                                    "APPVEYOR_REPO_BRANCH": "mybranch",
    #                                    "APPVEYOR_REPO_COMMIT": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertEquals(manager.get_commit_msg(), "msg more")
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")

    #     # Appveyor PULL REQUEST
    #     with tools.environment_append({"APPVEYOR": "1",
    #                                    "APPVEYOR_PULL_REQUEST_NUMBER": "1",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE": "msg",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE_EXTENDED": "more",
    #                                    "APPVEYOR_REPO_BRANCH": "mybranch",
    #                                    "APPVEYOR_REPO_COMMIT": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertIsNone(manager.get_branch())
    #         self.assertEquals(manager.get_commit_msg(), "msg more")
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")

    #     # Appveyor no extended
    #     with tools.environment_append({"APPVEYOR": "1",
    #                                    "APPVEYOR_PULL_REQUEST_NUMBER": "1",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE": "msg",
    #                                    "APPVEYOR_REPO_BRANCH": "mybranch",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertIsNone(manager.get_branch())
    #         self.assertEquals(manager.get_commit_msg(), "msg")

    # def test_circleci_instance(self):
    #     with tools.environment_append({"CIRCLECI": "1",
    #                                    "CIRCLE_BRANCH": "mybranch",
    #                                    "CIRCLE_SHA1": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertIsNotNone(manager.get_commit_msg())
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")
    #         self.assertEquals(manager.is_pull_request(), False)

    #     with tools.environment_append({"CIRCLECI": "1",
    #                                    "CIRCLE_BRANCH": "pull/35",
    #                                    "CIRCLE_SHA1": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    "CIRCLE_PULL_REQUEST": "https://github.com/org/repo/pull/35"
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertIsNone(manager.get_branch())
    #         self.assertIsNotNone(manager.get_commit_msg())
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")
    #         self.assertEquals(manager.is_pull_request(), True)

    # def test_gitlab_instance(self):
    #     with tools.environment_append({"GITLAB_CI": "1",
    #                                    "CI_BUILD_REF_NAME": "mybranch",
    #                                    "CI_COMMIT_TITLE": "foo bar",
    #                                    "CI_COMMIT_SHA": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertIsNotNone(manager.get_commit_msg())
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")

    # def test_jenkins_instance(self):
    #     with tools.environment_append({"JENKINS_URL": "1",
    #                                    "BRANCH_NAME": "mybranch",
    #                                    "GIT_COMMIT": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertIsNotNone(manager.get_commit_msg())
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")

    # def test_azure_instance(self):
    #     with tools.environment_append({"SYSTEM_TEAMFOUNDATIONCOLLECTIONURI": "https://dev.azure.com/",
    #                                    "BUILD_SOURCEVERSIONMESSAGE": "msg",
    #                                    "BUILD_SOURCEVERSION": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    "BUILD_SOURCEBRANCH": "mybranch",
    #                                    "BUILD_REASON": "manual",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch")
    #         self.assertEquals(manager.get_commit_msg(), "msg")
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")
    #         self.assertEquals(manager.is_pull_request(), False)

    #     with tools.environment_append({"SYSTEM_TEAMFOUNDATIONCOLLECTIONURI": "https://dev.azure.com/",
    #                                    "BUILD_SOURCEVERSIONMESSAGE": "msg",
    #                                    "BUILD_SOURCEVERSION": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    "BUILD_SOURCEBRANCH": "refs/heads/testing/version",
    #                                    "BUILD_REASON": "PullRequest",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "testing/version")
    #         self.assertEquals(manager.get_commit_msg(), "msg")
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")
    #         self.assertEquals(manager.is_pull_request(), True)

    #     with tools.environment_append({"SYSTEM_TEAMFOUNDATIONCOLLECTIONURI": "https://dev.azure.com/",
    #                                    "BUILD_SOURCEVERSIONMESSAGE": "msg",
    #                                    "BUILD_SOURCEVERSION": "506c89117650bb12252db26d35b8c2385411f175",
    #                                    "BUILD_SOURCEBRANCH": "refs/heads/stable/version",
    #                                    "BUILD_REASON": "IndividualCI",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "stable/version")
    #         self.assertEquals(manager.get_commit_msg(), "msg")
    #         self.assertEquals(manager.get_commit_id(), "506c89117650bb12252db26d35b8c2385411f175")
    #         self.assertEquals(manager.is_pull_request(), False)

    # def test_shippable_instance(self):
    #     shippable_env = {   "SHIPPABLE": "true",
    #                         "COMMIT_MESSAGE": "foobar [qux]",
    #                         "COMMIT": "98e984eacf4e3dfea431c8850c8c181a08e8cf3d",
    #                         "BRANCH": "testing/5.6.5",
    #                         "IS_GIT_TAG": "false",
    #                         "IS_PULL_REQUEST": "false"}
    #     with tools.environment_append(shippable_env):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), shippable_env["BRANCH"])
    #         self.assertEquals(manager.get_commit_msg(), shippable_env["COMMIT_MESSAGE"])
    #         self.assertEquals(manager.get_commit_id(), shippable_env["COMMIT"])
    #         self.assertEquals(manager.is_pull_request(), False)
    #         self.assertEquals(manager.is_tag(), False)

    #     shippable_env = {   "SHIPPABLE": "true",
    #                         "COMMIT_MESSAGE": "new tag",
    #                         "COMMIT": "98e984eacf4e3dfea431c8850c8c181a08e8cf3d",
    #                         "BRANCH": "release/5.6.5",
    #                         "IS_GIT_TAG": "true",
    #                         "IS_PULL_REQUEST": "true"}
    #     with tools.environment_append(shippable_env):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), shippable_env["BRANCH"])
    #         self.assertEquals(manager.get_commit_msg(), shippable_env["COMMIT_MESSAGE"])
    #         self.assertEquals(manager.get_commit_id(), shippable_env["COMMIT"])
    #         self.assertEquals(manager.is_pull_request(), True)
    #         self.assertEquals(manager.is_tag(), True)

    # def test_github_actions_instance(self):
    #     gha_env = {"GITHUB_ACTIONS": "true",
    #                "GITHUB_SHA": "98e984eacf4e3dfea431c8850c8c181a08e8cf3d",
    #                "GITHUB_REF": "testing/5.6.5",
    #                "GITHUB_BASE_REF": "testing/5.6.5",
    #                "GITHUB_EVENT_NAME": "push"}
    #     with tools.environment_append(gha_env):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), gha_env["GITHUB_REF"])
    #         self.assertEquals(manager.get_commit_id(), gha_env["GITHUB_SHA"])
    #         self.assertEquals(manager.is_pull_request(), False)

    #     gha_env = {"GITHUB_ACTIONS": "true",
    #                "GITHUB_SHA": "98e984eacf4e3dfea431c8850c8c181a08e8cf3d",
    #                "GITHUB_REF": "quick_fix",
    #                "GITHUB_BASE_REF": "testing/5.6.5",
    #                "GITHUB_EVENT_NAME": "pull_request"}
    #     with tools.environment_append(gha_env):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), gha_env["GITHUB_BASE_REF"])
    #         self.assertEquals(manager.get_commit_id(), gha_env["GITHUB_SHA"])
    #         self.assertEquals(manager.is_pull_request(), True)

    #     gha_env = {"GITHUB_ACTIONS": "true",
    #                "GITHUB_REF": "refs/heads/testing",
    #                "GITHUB_EVENT_NAME": "push"}
    #     with tools.environment_append(gha_env):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "testing")

    # def test_build_policy(self):
    #     # Travis
    #     with tools.environment_append({"TRAVIS": "1",
    #                                    "TRAVIS_COMMIT_MESSAGE":
    #                                        "This is a great commit [build=outdated] End."}):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_commit_build_policy(), ["outdated"])
    #         self.assertEquals(manager.get_commit_msg(), "This is a great commit "
    #                                                     "[build=outdated] End.")

    #     with tools.environment_append({"TRAVIS": "1",
    #                                    "TRAVIS_COMMIT_MESSAGE":
    #                                        "This is a great commit [build=all] End."}):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_commit_build_policy(), ["all"])
    #         self.assertEquals(manager.get_commit_msg(), "This is a great commit "
    #                                                     "[build=all] End.")
    #     # Appveyor
    #     with tools.environment_append({"APPVEYOR": "1",
    #                                    "APPVEYOR_PULL_REQUEST_NUMBER": "1",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE": "msg",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE_EXTENDED":
    #                                        "more [build=missing] ",
    #                                    "APPVEYOR_REPO_BRANCH": "mybranch",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_commit_build_policy(), ["missing"])

    #     # Complex messages
    #     m = "double travis pages again due to timeout, travis taking longer " \
    #         "now [skip appveyor] [build=missing]"
    #     with tools.environment_append({"TRAVIS": "1",
    #                                    "TRAVIS_COMMIT_MESSAGE": m}):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_commit_build_policy(), ["missing"])

    #     # multiple build policies
    #     with tools.environment_append({"APPVEYOR": "1",
    #                                    "APPVEYOR_PULL_REQUEST_NUMBER": "1",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE": "msg",
    #                                    "APPVEYOR_REPO_COMMIT_MESSAGE_EXTENDED":
    #                                        "more [build=missing] [build=pattern] ",
    #                                    "APPVEYOR_REPO_BRANCH": "mybranch",
    #                                    }):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_commit_build_policy(), ["missing", "pattern"])

    # def test_bamboo_env_vars(self):
    #     self.assertIsNone(os.getenv('CONAN_LOGIN_USERNAME'))

    #     with tools.environment_append({"bamboo_buildNumber": "xx",
    #                                    "bamboo_planRepository_branch": "mybranch",
    #                                    "bamboo_CONAN_LOGIN_USERNAME": "bamboo",
    #                                    "bamboo_CONAN_USER_VAR": "bamboo",
    #                                    "CONAN_USER_VAR": "foobar"}):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch") # checks that manager is Bamboo

    #         self.assertEquals(os.getenv('CONAN_LOGIN_USERNAME'), "bamboo")
    #         self.assertEquals(os.getenv('CONAN_USER_VAR'), "foobar")

    #     with tools.environment_append({"bamboo_buildNumber": "xx",
    #                                    "bamboo_planRepository_branch": "mybranch",
    #                                    "BAMBOO_CONAN_LOGIN_USERNAME": "bamboo",
    #                                    "BAMBOO_CONAN_USER_VAR": "bamboo",
    #                                    "CONAN_USER_VAR": "foobar"}):
    #         manager = CIManager(self.printer)
    #         self.assertEquals(manager.get_branch(), "mybranch") # checks that manager is Bamboo

    #         self.assertEquals(os.getenv('CONAN_LOGIN_USERNAME'), "bamboo")
    #         self.assertEquals(os.getenv('CONAN_USER_VAR'), "foobar")



import os
from collections import namedtuple

import mock

from conans import tools
from conans.model.ref import ConanFileReference
from cpt.test.utils.test_files import temp_folder
from conans.util.files import save
from conans.model.version import Version
from cpt import get_client_version


class MockRunner(object):

    def __init__(self):
        self.reset()
        self.output = ""

    def reset(self):
        self.calls = []

    def __call__(self, command):
        self.calls.append(command)
        return 0


class MockConanCache(object):

    def __init__(self, *args, **kwargs):
        _base_dir = temp_folder()
        self.default_profile_path = os.path.join(_base_dir, "profiles", "default")
        self.profiles_path = os.path.join(_base_dir, "profiles")
        self.new_config_path = os.path.join(_base_dir, "global.conf")
        if not os.path.exists(self.profiles_path):
            os.mkdir(self.profiles_path)

Action = namedtuple("Action", "name args kwargs")

class MockConanAPI(object):

    def __init__(self):
        self.calls = []
        self._client_cache = self._cache = MockConanCache()
        self.app = mock.Mock()
        self.app.cache = self._client_cache

    def create(self, *args, **kwargs):
        reference = ConanFileReference(kwargs["name"], kwargs["version"], kwargs["user"], kwargs["channel"])
        self.calls.append(Action("create", args, kwargs))
        return {
            "installed": [
               {
                  "packages": [
                     {
                        "id": "227fb0ea22f4797212e72ba94ea89c7b3fbc2a0c",
                        "built": True
                     }
                  ],
                  "recipe": {
                     "id": str(reference)
                  },
               }]}

    def create_profile(self, *args, **kwargs):
        save(os.path.join(self._client_cache.profiles_path, args[0]), "[settings]")
        self.calls.append(Action("create_profile", args, kwargs))

    def config_install(self, *args, **kwargs):
        self.calls.append(Action("config_install", args, kwargs))

    def remote_list(self, *args, **kwargs):
        self.calls.append(Action("remote_list", args, kwargs))
        return []

    def remote_add(self, *args, **kwargs):
        self.calls.append(Action("remote_add", args, kwargs))
        return args[0]

    def authenticate(self, *args, **kwargs):
        self.calls.append(Action("authenticate", args, kwargs))

    def upload(self, *args, **kwargs):
        self.calls.append(Action("upload", args, kwargs))

    def get_profile_from_call_index(self, number):
        call = self.calls[number]
        return self.get_profile_from_call(call)

    def get_profile_from_call(self, call):
        if call.name != "create":
            raise Exception("Invalid test, not contains a create: %s" % self.calls)
        from conans.client.profile_loader import read_profile
        conan_version = get_client_version()
        if Version(conan_version) < Version("1.12.0"):
            profile_name = call.kwargs["profile_name"]
        else:
            profile_name = call.kwargs["profile_names"][0]
        tools.replace_in_file(profile_name, "include", "#include")
        return read_profile(profile_name, os.path.dirname(profile_name), None)[0]

    def reset(self):
        self.calls = []

    def get_creates(self):
        return [call for call in self.calls if call.name == "create"]

    def assert_tests_for(self, indexes):
        creates = self.get_creates()
        for create_index, index in enumerate(indexes):
            profile = self.get_profile_from_call(creates[create_index])
            assert("os%s" % index == profile.settings["os"])


class MockCIManager(object):

    def __init__(self, current_branch=None, build_policy=None, skip_builds=False, is_pull_request=False, is_tag=False):
        self._current_branch = current_branch
        self._build_policy = [build_policy] if build_policy != None and not isinstance(build_policy, list) else build_policy
        self._skip_builds = skip_builds
        self._is_pr = is_pull_request
        self._is_tag = is_tag

    def get_commit_build_policy(self):
        return self._build_policy

    def skip_builds(self):
        return self._skip_builds

    def is_pull_request(self):
        return self._is_pr

    def is_tag(self):
        return self._is_tag

    def get_branch(self):
        return self._current_branch
